;***************************************************************
;  STARTUP.ASM  fr EVB-4088                                   *
;  mit Philips LPC4088 und externem SDRAM  16 MB               *
; (Micron MT48LC4M32B2-7 = 128 MBit = 1M x 32 x 4 Bnke)       *
;  Clock=143MHz, CI=3,  5.5ns, setup=2ns, hold=1ns             *
;  Refreshcount=4K                                             *
;  auf dem Chip sind 512K Flash und 96K RAM in div. Blcken    *
;  die CPU kann max. 120 MHz, luft erstmal mit 120 MHz,       *
;  Quarz = 6 MHz, war zuerst 10 MHz                            *
;***************************************************************

     EXPORT     Errors          ; DWORD
     EXPORT     ErrAddr
     EXPORT     Manifest
     EXPORT     Hard_Reset
     EXPORT     Soft_Reset
     EXPORT     StartOfRam
     EXPORT     EndOfRam
     EXPORT     abort_me
     EXPORT     CS_Area0
     EXPORT     CS_Area1
     EXPORT     CS_Area2
     EXPORT     CS_Area3
     EXPORT     Cpt
     EXPORT     von
     EXPORT     bis
     EXPORT     kzi
     EXPORT     Kommandozeile
     EXPORT     CgBWSP
     EXPORT     ExtRamBegin

     EXPORT     IOCON_P0
     EXPORT     IOCON_P1
     EXPORT     IOCON_P2
     EXPORT     IOCON_P3
     EXPORT     IOCON_P4
     EXPORT     IOCON_P5

     EXPORT     LCD_PAL
     EXPORT     LCD_CRSR_IMG


; interne Resourcen:
; ==================
; ROM-Bereich: 00000000 .. 0FFFFFFF
; 512 K Flash     ab 0..7FFFFh

; RAM-Bereiche:   10000000..2007FFFF
; hier 64 K RAM   10000000..1000FFFFh
; darin:
;   diverse Householding-Variablen
;   darunter der Stack
;   darunter das Ende des frei nutzbaren RAMs

; AHB-Peripherie: 20080000 .. 3FFFFFFF
; 16 K RAM        ab 20000000h..20001FFFh  RAM Bank 0
; 16 K RAM        ab 20002000h..20003FFFh  RAM Bank 0
; 32 K RAM        ab 20004000h..20007FFFh  RAM Bank 1

; APB (Peripherie) 40000000 .. 7FFFFFFF

; externe Resourcen:
; ==================
; 4 Speicherbnke fr statischen Speicher jeweils 64 MB adressierbar:
CS_Area0   EQU    0x80000000   ; 80000000h..83FFFFFFh  
CS_Area1   EQU    0x90000000   ; 90000000h..93FFFFFFh  
CS_Area2   EQU    0x98000000   ; 98000000h..9BFFFFFFh  
CS_Area3   EQU    0x9C000000   ; 9C000000h..9FFFFFFFh  

; 4 Speicherbnke fr dynamischen Speicher jeweils 256 MB
DYCS0_Area EQU    0xA0000000   ; A0000000h..AFFFFFFFh   hier ist SDRAM 16 MB angeschlossen
DYCS1_Area EQU    0xB0000000 
DYCS2_Area EQU    0xC0000000 
DYCS3_Area EQU    0xD0000000 

; Cortex M4 privater Bereich
; E0000000..E00FFFFF

; Port-Steuerregister
IOCON_P0   EQU    0x4002C000
IOCON_P1   EQU    0x4002C080
IOCON_P2   EQU    0x4002C100
IOCON_P3   EQU    0x4002C180
IOCON_P4   EQU    0x4002C200
IOCON_P5   EQU    0x4002C280

; LCD-Palettenfeld und Cursor-Zeiger-Feld
LCD_PAL       EQU    0x20088200
LCD_CRSR_IMG  EQU    0x20088800


ExtRamBegin EQU     0xA0000000

; Bildwiederholspeicher fr TFT definieren (letztes MB im ext. RAM)
CgBWSP      EQU     0xA0F00000


;----------------------- LCD Controller Register -------------------------------
LCD_CTRL    EQU     0x20088018

;----------------------- Power Control Register --------------------------------
; PCON = diverse Powerdown-Modi. 0=normal
PCON       EQU     0x400FC0C0  

; PCONP = Power fr diverse Pripherie
PCONP      EQU     0x400FC0C4 

;----------------------- Flash Accelerator Module Definitions ------------------
FLASHCFG   EQU     0x400FC000

; bis hier----

;------------- Taktversorgung --------------------------------------------------
; System Control and Status:
; Bit 0: 0=alle Ports klassisch, 1=P0..1 per FIOxxx
; Bit 1: ExtMemController Resetverhalten
; Bit 2: ob EMC Burst
; Bit 3: ob MCIPWR Pin log hi ode lo
; Bit 4: 0=Main Oszi 1..20 MHz, 1=15..25MHz
; Bit 5: 0=Main Oszi aus, 1=Main Oszi ein
; Bit 6: R/O  1= Main Oszi luft
SCS             EQU     0xE01FC1A0  ; also erstmal = 0x21 setzen (Oszi ein + FIO an P0,P1)
; Taktquelle selektieren
CLKSRCSEL       EQU     0xE01FC10C  ; 0=RC, 1=Main, 2=RTC, alles andere verboten


; VPBDIV = Taktteiler zwischen AMBA- und Peripherie-Bus (VPB)
; nach Reset: VPB arbeitet auf 1/4 des AMBA-Taktes

VPBDIV          EQU     0xE01FC100  ; VPBDIV Addresse
VPBDIV_Val      EQU     0x00000002  ; 0 = 1/4; 1 = 1:1; 2 = 1/2

; Phase Locked Loop (PLL) definitions
PLL_BASE        EQU     0xE01FC080  ; PLL Base Address
PLLCON_OFS      EQU     0x00        ; PLL Connect   Bit 0: 1=enable, Bit 1: 1=connect
PLLCFG_OFS      EQU     0x04        ; PLL Config    Bits [14..0]=MSEL, [23..16]=NSEL
PLLSTAT_OFS     EQU     0x08        ; PLL Status
PLLFEED_OFS     EQU     0x0C        ; PLL Feed
PLLCPUCFG_OFS   EQU     0x84        ; fr den CPU-Takt
PLLUSBCFG_OFS   EQU     0x88        ; fr den USB-Takt
PLLCLKSRC_OFS   EQU     0x8C        ; 0=RC, 1=Main, 2=RTC, alles andere verboten
PLLPCLKSEL0_OFS EQU     0x128       ; @ 0xE01FC1A8
PLLPCLKSEL1_OFS EQU     0x12C       ; @ 0xE01FC1AC

; Clock-Einschalter fr diverse Peripherie, jeweils 2 Bit, aus geteilter CCLK (CPU Takt)
; wobei die jeweilige PCLK wie folgt ist (ACHTUNG: Fehler im Manual??
; 00 = 0 =   (Manual = CCLK/4)
; 01 = 1 =   (Manual = CCLK  )
; 10 = 2 =   (Manual = CCLK/2)
; 11 = 3 =   (Manual = CCLK/8)   (mit Ausnahme CAN..)
pWD    EQU     (3<<0)        ; Watchdog    1/8 reicht
pT0    EQU     (2<<2)        ; Timer 0     1/2
pT1    EQU     (2<<4)        ; Timer 1     1/2
pU0    EQU     (2<<6)        ; Uart  0     1/2
pU1    EQU     (2<<8)        ; Uart  1     1/2
pP0    EQU     (2<<10)       ; Pwm   0     1/2
pP1    EQU     (2<<12)       ; Pwm   1     1/2
pI0    EQU     (2<<14)       ; I2C   0     1/2
pSP    EQU     (2<<16)       ; SPI
pRT    EQU     (3<<18)       ; RTC     (hier nicht 01 mglich)
pS1    EQU     (2<<20)       ; SSP   1
pDA    EQU     (2<<22)       ; DAC
pAD    EQU     (2<<24)       ; ADC
pC1    EQU     (2<<26)       ; CAN   1
pC2    EQU     (2<<28)       ; CAN   2
pACF   EQU     (2<<30)       ; CAN Filterei


pBR    EQU     (3<<0)        ; fr batt. gesttzen RAM
pGP    EQU     (1<<2)        ; GPIO
pPC    EQU     (2<<4)        ; Pin Connect Block
pI1    EQU     (2<<6)        ; I2C   1     1/8
p_     EQU     (2<<8)        ; unused
pS0    EQU     (2<<10)       ; SSP   0
pT2    EQU     (2<<12)       ; Timer 2
pT3    EQU     (2<<14)       ; Timer 3
pU2    EQU     (2<<16)       ; Uart  2
pU3    EQU     (2<<18)       ; Uart  3
pI2    EQU     (2<<20)       ; I2C   2
pIS    EQU     (2<<22)       ; I2S        1:1
pMCI   EQU     (2<<24)       ; MCI        1:1   stimmt auch nicht
p__    EQU     (0<<26)       ; unused
pSYS   EQU     (0<<28)       ; System Control Block
p___   EQU     (0<<30)       ; unused

PCLKSEL0_Val    EQU     pWD + pT0 + pT1 + pU0 + pU1 + pP0 + pP1 + pI0 + pSP + pRT + pS1 + pDA  + pAD + pC1 + pC2 + pACF
PCLKSEL1_Val    EQU     pBR + pGP + pPC + pI1 + pS0 + pT2 + pT3 + pU2 + pU3 + pI2 + pIS + pMCI + pSYS


PLLCON_PLLE     EQU     (1<<0)      ; PLL Enable
PLLCON_PLLC     EQU     (1<<1)      ; PLL Connect
PLLSTAT_PLLE    EQU     (1<<24)     ; PLL Enable Status
PLLSTAT_PLLC    EQU     (1<<25)     ; PLL Connect Status
PLLSTAT_PLOCK   EQU     (1<<26)     ; PLL Lock Status

; Fvco = Fquarz * 2 * (M) / (N);
; bzw.
; Fvco = Fquarz * 2 * (MSEL+1) / (NSEL+1);

; M= 6..512, sowie ausgewhlte Werte von 4272 bis 24170
; N= 1..32
; Der VCO kann von 275 bis 550 MHz schwingen.. eigentlich.
; lt. Errata aber tatschlich nur bis ca. 290 MHz
; also 480 MHz geht wohl nicht, nehmen wir 288 MHz (Quarz=6MHz * 48)
; Fquarz ist hier 6(war 10) MHz. Wir wollen erstmal Fvco=288 MHz erzeugen
; also N=1 und M = 24 ergibt 6MHz * 2 * 24 / 1 --> 288MHz
MSEL              EQU    23         ; M=24
NSEL              EQU     0         ; N=1  , macht 2*24*10MHz / 1 = 288 MHz

PLLCFG_Val        EQU    (NSEL << 16) + MSEL




; RAM und Stack
; =============
USER_SS           EQU     8192            ; 8 K Stack sollte erstmal ausreichen

Top_Ram           EQU     0x10010000      ; der interne RAM geht von 10000000 bis 1000FFFF
Errors            EQU     Top_Ram - 4     ; Bitfeld zum Merken von Fehlern etc. und zugleich..
ErrAddr           EQU     Errors  - 4     ; Adresse des letzten Fehlers
Cpt               EQU     ErrAddr - 4     ; allg. Pointer
von               EQU     Cpt - 4         ; allg. Pointer
bis               EQU     von - 4         ; allg. Pointer
kzi               EQU     bis - 4         ; Index fr Kommandozeile
Kommandozeile     EQU     kzi   - 128     ; Speicherbereich fr serielle Kommunikation

Top_Stack         EQU     Kommandozeile - 4
__initial_sp      EQU     Kommandozeile - 4

StartOfRam        EQU     0x40000000
EndOfRam          EQU     Top_Stack - USER_SS




                PRESERVE8
                THUMB


; Vector Table Mapped to Address 0 at Reset

                AREA    RESET, CODE, READONLY
                EXPORT  __Vectors

__Vectors       DCD             __initial_sp              ; Top of Stack
                DCD             Hard_Reset                ; Reset Handler
                DCD             NMI_Handler               ; NMI Handler
                DCD             HardFault_Handler         ; Hard Fault Handler
                DCD             MemManage_Handler         ; MPU Fault Handler
                DCD             BusFault_Handler          ; Bus Fault Handler
                DCD             UsageFault_Handler        ; Usage Fault Handler
                DCD             0                         ; 0xEFFFF39E  Reserved- vector sum
                DCD             0                         ; Reserved
                DCD             0                         ; Reserved
                DCD             0                         ; Reserved
                DCD             SVC_Handler               ; SVCall Handler
                DCD             DebugMon_Handler          ; Debug Monitor Handler
                DCD             0                         ; Reserved
                DCD             PendSV_Handler            ; PendSV Handler
                DCD             SysTick_Handler           ; SysTick Handler

                ; External Interrupts
                DCD             WDT_IRQHandler            ; 16: Watchdog Timer
                DCD             TIMER0_IRQHandler         ; 17: Timer0
                DCD             TIMER1_IRQHandler         ; 18: Timer1
                DCD             TIMER2_IRQHandler         ; 19: Timer2
                DCD             TIMER3_IRQHandler         ; 20: Timer3
                DCD             UART0_IRQHandler          ; 21: UART0
                DCD             UART1_IRQHandler          ; 22: UART1
                DCD             UART2_IRQHandler          ; 23: UART2
                DCD             UART3_IRQHandler          ; 24: UART3
                DCD             PWM1_IRQHandler           ; 25: PWM1
                DCD             I2C0_IRQHandler           ; 26: I2C0
                DCD             I2C1_IRQHandler           ; 27: I2C1
                DCD             I2C2_IRQHandler           ; 28: I2C2
                DCD             0                         ; 29: reserved, not for SPIFI anymore
                DCD             SSP0_IRQHandler           ; 30: SSP0
                DCD             SSP1_IRQHandler           ; 31: SSP1
                DCD             PLL0_IRQHandler           ; 32: PLL0 Lock (Main PLL)
                DCD             RTC_IRQHandler            ; 33: Real Time Clock
                DCD             EINT0_IRQHandler          ; 34: External Interrupt 0
                DCD             EINT1_IRQHandler          ; 35: External Interrupt 1
                DCD             EINT2_IRQHandler          ; 36: External Interrupt 2
                DCD             EINT3_IRQHandler          ; 37: External Interrupt 3
                DCD             ADC_IRQHandler            ; 38: A/D Converter
                DCD             BOD_IRQHandler            ; 39: Brown-Out Detect
                DCD             USB_IRQHandler            ; 40: USB
                DCD             CAN_IRQHandler            ; 41: CAN
                DCD             DMA_IRQHandler            ; 42: General Purpose DMA
                DCD             I2S_IRQHandler            ; 43: I2S
                DCD             ENET_IRQHandler           ; 44: Ethernet
                DCD             MCI_IRQHandler            ; 45: SD/MMC card I/F
                DCD             MCPWM_IRQHandler          ; 46: Motor Control PWM
                DCD             QEI_IRQHandler            ; 47: Quadrature Encoder Interface
                DCD             PLL1_IRQHandler           ; 48: PLL1 Lock (USB PLL)
                DCD             USBActivity_IRQHandler    ; 49: USB Activity interrupt to wakeup
                DCD             CANActivity_IRQHandler    ; 50: CAN Activity interrupt to wakeup
                DCD             UART4_IRQHandler          ; 51: UART4
                DCD             SSP2_IRQHandler           ; 52: SSP2
                DCD             LCD_IRQHandler            ; 53: LCD
                DCD             GPIO_IRQHandler           ; 54: GPIO
                DCD             PWM0_IRQHandler           ; 55: PWM0
                DCD             EEPROM_IRQHandler         ; 56: EEPROM
                DCD             CMP0_IRQHandler           ; 57: Komparator_0 ? nicht hier
                DCD             CMP1_IRQHandler           ; 58: Komparator_1 ? nicht hier

;                ENTRY
;
Manifest        DCB    "EVB4088 Evalboard",0
                ALIGN


;----------------------------------------------------------------------
; Hier geht es nach dem Reset los.
Hard_Reset
     LDR     R0, = LCD_CTRL         ; LCD ausschalten
     MOV     R3,#0
     STR     R3, [R0]

     LDR     R0, = SCS              ; Main Oszillator einschalten
     LDR     R3, = 0x29             ; Oszi 1..20MHz, ein, Port0+1-->FIOxxx
     STR     R3, [R0]

OSC_Loop
     LDR     R3, [R0]               ; Wait for main osc stabilize
     ANDS    R3, R3, #0x40
     BEQ     OSC_Loop

; PLL anwerfen
     LDR     R0, =PLL_BASE          ; Configure and Enable PLL
     MOV     R1, #0xAA              ; Entsperrcode 1
     MOV     R2, #0x55              ; Entsperrcode 2

     MOV     R3, #1                 ; 1=Main Oszi als Taktquelle
     STR     R3, [R0, #PLLCLKSRC_OFS]

     LDR     R3, =PLLCFG_Val        ; Wert fr PLLCFG
     STR     R3, [R0, #PLLCFG_OFS]  ; PLLCFG setzen

     MOV     R3, #PLLCON_PLLE       ; Enable-Sequenz
     STR     R3, [R0, #PLLCON_OFS]
     STR     R1, [R0, #PLLFEED_OFS]
     STR     R2, [R0, #PLLFEED_OFS]

PLL_Loop                            ; Wait until PLL Locked
     LDR     R3, [R0, #PLLSTAT_OFS]
     ANDS    R3, R3, #PLLSTAT_PLOCK
     BEQ     PLL_Loop


;  Setup CPU clock divider  (Fvco = 288 MHz)
     MOV     R3, #3                 ; 288 MHZ / (3+1) --> 72 MHz fr die CPU
     STR     R3, [R0, #PLLCPUCFG_OFS]

;  Setup USB clock divider  (Fvco = 288 MHz)
     MOV     R3, #5                 ; 288 MHz / (5+1) --> 48 MHz fr den USB
     STR     R3, [R0, #PLLUSBCFG_OFS]

;  Setup Peripheral Clock
     LDR     R3, =PCLKSEL0_Val
     STR     R3, [R0, #PLLPCLKSEL0_OFS]
     LDR     R3, =PCLKSEL1_Val
     STR     R3, [R0, #PLLPCLKSEL1_OFS]

;  Switch to PLL Clock
     MOV     R3, #(PLLCON_PLLE  :OR: PLLCON_PLLC)
     STR     R3, [R0, #PLLCON_OFS]
     STR     R1, [R0, #PLLFEED_OFS]
     STR     R2, [R0, #PLLFEED_OFS]

;  Memory Accelerator Module aufsetzen

;  internen RAM ablschen
     LDR     R0, =StartOfRam
     MOV     R1, #0
     LDR     R2, =Top_Ram
iloop
     STR     R1, [R0], #4
     CMP     R0, R2                 ; RAM, Stacks und Kommandozeile
     BLT     iloop

     LDR     R0, =Errors
     STR     R1, [R0]               ;  Fehlerbits lschen

     B       Restart


; Wiedereintrittspunkt fr Soft-Reset
     EXPORT  Soft_Reset
Soft_Reset
     MOV     R1, #istWarmstart

; Error-Bit in R1
Restart
     LDR     R0, =ErrAddr
     STR     LR,[R0]
     LDR     R0, =Errors
     LDR     R2,[R0]
     ORR     R2,R2,R1
     STR     R2,[R0]


; Enter the C code
_main
__main
       EXPORT _main
       EXPORT __main
       EXPORT  __raise
       EXPORT  _sys_exit

       IMPORT  sysconfig
       IMPORT  main

       LDR     R0, =sysconfig
       BLX     R0

       LDR     R0, =main
       BLX     R0

_sys_exit
       B        Soft_Reset

;=============================================================================
; Interrupt-Behandlungsroutinen.
;=============================================================================
; Restart-Grnde
; ==============
istKaltstart      EQU   0
istWarmstart      EQU   1
istRaise          EQU   2
istHardFaultErr   EQU   4
istMemManageErr   EQU   8
istBusFaultErr    EQU  16
istUsageFaultErr  EQU  32
istWrongSwi       EQU  64
istNMI            EQU 128
istWeakInt        EQU 256
istDebugmonErr    EQU 512
istPendSVErr      EQU 1024
istSysTickErr     EQU 2048
; alles >= 0x80000000 ist Abort


;-----------------------------------------------------------------------------
NMI_Handler        PROC
                   EXPORT  NMI_Handler          [WEAK]
                   LDR     R1, =istNMI
                   B       Restart
                   ENDP

;-----------------------------------------------------------------------------
HardFault_Handler  PROC
                   EXPORT  HardFault_Handler    [WEAK]
                   LDR    R0, [R13, #24]        ; Get previous PC
                   LDRH   R1, [R0]              ; Get instruction
                   LDR    R2, =0xBEAB           ; The special BKPT instruction
                   CMP    R1, R2                ; Test if the instruction at previous PC is BKPT
                   BNE    HardFault_Restart     ; Not BKPT

                   ADDS   R0, #4                ; Skip BKPT and next line
                   STR    R0, [R13, #24]        ; Save previous PC
                   BX     LR

HardFault_Restart  LDR     R1, =2               ; Bit 1 --> war Hardfault
                   B       Restart
                   ENDP

;-----------------------------------------------------------------------------
MemManage_Handler  PROC                        ; MPU Fault Handler
                   EXPORT  MemManage_Handler   [WEAK]
                   LDR     R1, =istMemManageErr
                   B       Restart
                   ENDP

;-----------------------------------------------------------------------------
BusFault_Handler   PROC                        ;  Bus Fault Handler
                   EXPORT  BusFault_Handler    [WEAK]
                   LDR     R1, =istBusFaultErr
                   B       Restart
                   ENDP

;-----------------------------------------------------------------------------
UsageFault_Handler PROC                        ;  Usage Fault Handler
                   EXPORT  UsageFault_Handler  [WEAK]
                   LDR     R1, =istUsageFaultErr
                   B       Restart
                   ENDP

;-----------------------------------------------------------------------------
DebugMon_Handler   PROC                        ;  DebugMonitor Handler
                   EXPORT  DebugMon_Handler    [WEAK]
                   LDR     R1, =istDebugmonErr
                   B       Restart
                   ENDP

;-----------------------------------------------------------------------------
PendSV_Handler     PROC                        ;  PendSV Handler
                   EXPORT  PendSV_Handler      [WEAK]
                   LDR     R1, =istPendSVErr
                   B       Restart
                   ENDP

;-----------------------------------------------------------------------------
SysTick_Handler    PROC                        ;  Cortex System Tick Handler
                   EXPORT  SysTick_Handler     [WEAK]
                   LDR     R1, =istSysTickErr
                   B       Restart
                   ENDP

;-----------------------------------------------------------------------------
; Software-Interrupt.
; Achtung! Die aufgerufenen Teile laufen im Supervisor-Modus und drfen
; ihrerseits KEINE Software-Interrupts verwenden!!
; SWI-Handler a la Keil
T_Bit           EQU     0x20


                EXPORT  SVC_Handler

SVC_Handler
                PRESERVE8
                TST     LR,#4                   ; Called from Handler Mode?
                MRSNE   R12,PSP                 ; Yes, use PSP
                MOVEQ   R12,SP                  ; No, use MSP
                LDR     R12,[R12,#24]           ; Read Saved PC from Stack
                LDRH    R12,[R12,#-2]           ; Load Halfword
                BICS    R12,R12,#0xFF00         ; Extract SVC Number

                PUSH    {R4,LR}                 ; Save Registers
                LDR     LR,=SVC_Count
                LDR     LR,[LR]
                CMP     R12,LR
                BHS     SVC_Dead                ; Overflow
                LDR     LR,=SVC_Table
                LDR     R12,[LR,R12,LSL #2]     ; Load SVC Function Address
                BLX     R12                     ; Call SVC Function

                POP     {R4,LR}
                TST     LR,#4
                MRSNE   R12,PSP
                MOVEQ   R12,SP
                STM     R12,{R0-R3}             ; Function return values
                BX      LR                      ; RETI

SVC_Dead        MOV     R1, #istWrongSwi       ; None Existing SWI
                B       Restart


SVC_Cnt         EQU    (SVC_End-SVC_Table)/4
SVC_Count       DCD     SVC_Cnt

SVC_Table
                DCD  SVC_Dead                   ; 0   die ersten werden freigehalten
                DCD  SVC_Dead                   ; 1   wegen evtl. RTOS
                DCD  SVC_Dead                   ; 2
                DCD  SVC_Dead                   ; 3
                DCD  SVC_Dead                   ; 4
                DCD  SVC_Dead                   ; 5
                DCD  SVC_Dead                   ; 6
                DCD  SVC_Dead                   ; 7

; ab hier OS-Funktionen
                DCD  Soft_Reset                 ; 8 --> Restart


SVC_End

;-----------------------------------------------------------------------------
; nur zum Export. Wird von mir nicht aktiv benutzt.
__raise
       MOV      R1, #istRaise
       B        Restart

       EXPORT   abort_me
abort_me
       ORR      R1,R0,#0x80000000      ; Param nach R1
       B        Restart

;-----------------------------------------------------------------------------
Default_Handler    PROC
                EXPORT    WDT_IRQHandler         [WEAK]
                EXPORT    TIMER0_IRQHandler      [WEAK]
                EXPORT    TIMER1_IRQHandler      [WEAK]
                EXPORT    TIMER2_IRQHandler      [WEAK]
                EXPORT    TIMER3_IRQHandler      [WEAK]
                EXPORT    UART0_IRQHandler       [WEAK]
                EXPORT    UART1_IRQHandler       [WEAK]
                EXPORT    UART2_IRQHandler       [WEAK]
                EXPORT    UART3_IRQHandler       [WEAK]
                EXPORT    PWM1_IRQHandler        [WEAK]
                EXPORT    I2C0_IRQHandler        [WEAK]
                EXPORT    I2C1_IRQHandler        [WEAK]
                EXPORT    I2C2_IRQHandler        [WEAK]
                EXPORT    SSP0_IRQHandler        [WEAK]
                EXPORT    SSP1_IRQHandler        [WEAK]
                EXPORT    PLL0_IRQHandler        [WEAK]
                EXPORT    RTC_IRQHandler         [WEAK]
                EXPORT    EINT0_IRQHandler       [WEAK]
                EXPORT    EINT1_IRQHandler       [WEAK]
                EXPORT    EINT2_IRQHandler       [WEAK]
                EXPORT    EINT3_IRQHandler       [WEAK]
                EXPORT    ADC_IRQHandler         [WEAK]
                EXPORT    BOD_IRQHandler         [WEAK]
                EXPORT    USB_IRQHandler         [WEAK]
                EXPORT    CAN_IRQHandler         [WEAK]
                EXPORT    DMA_IRQHandler         [WEAK]
                EXPORT    I2S_IRQHandler         [WEAK]
                EXPORT    ENET_IRQHandler        [WEAK]
                EXPORT    MCI_IRQHandler         [WEAK]
                EXPORT    MCPWM_IRQHandler       [WEAK]
                EXPORT    QEI_IRQHandler         [WEAK]
                EXPORT    PLL1_IRQHandler        [WEAK]
                EXPORT    USBActivity_IRQHandler [WEAK]
                EXPORT    CANActivity_IRQHandler [WEAK]
                EXPORT    UART4_IRQHandler       [WEAK]
                EXPORT    SSP2_IRQHandler        [WEAK]
                EXPORT    LCD_IRQHandler         [WEAK]
                EXPORT    GPIO_IRQHandler        [WEAK]
                EXPORT    PWM0_IRQHandler        [WEAK]
                EXPORT    EEPROM_IRQHandler      [WEAK]
                EXPORT    CMP0_IRQHandler        [WEAK]
                EXPORT    CMP1_IRQHandler        [WEAK]

WDT_IRQHandler         
TIMER0_IRQHandler         ; 17: Timer0
TIMER1_IRQHandler         ; 18: Timer1
TIMER2_IRQHandler         ; 19: Timer2
TIMER3_IRQHandler         ; 20: Timer3
UART0_IRQHandler          ; 21: UART0
UART1_IRQHandler          ; 22: UART1
UART2_IRQHandler          ; 23: UART2
UART3_IRQHandler          ; 24: UART3
PWM1_IRQHandler           ; 25: PWM1
I2C0_IRQHandler           ; 26: I2C0
I2C1_IRQHandler           ; 27: I2C1
I2C2_IRQHandler           ; 28: I2C2
SSP0_IRQHandler           ; 30: SSP0
SSP1_IRQHandler           ; 31: SSP1
PLL0_IRQHandler           ; 32: PLL0 Lock (Main PLL)
RTC_IRQHandler            ; 33: Real Time Clock
EINT0_IRQHandler          ; 34: External Interrupt 0
EINT1_IRQHandler          ; 35: External Interrupt 1
EINT2_IRQHandler          ; 36: External Interrupt 2
EINT3_IRQHandler          ; 37: External Interrupt 3
ADC_IRQHandler            ; 38: A/D Converter
BOD_IRQHandler            ; 39: Brown-Out Detect
USB_IRQHandler            ; 40: USB
CAN_IRQHandler            ; 41: CAN
DMA_IRQHandler            ; 42: General Purpose DMA
I2S_IRQHandler            ; 43: I2S
ENET_IRQHandler           ; 44: Ethernet
MCI_IRQHandler            ; 45: SD/MMC card I/F
MCPWM_IRQHandler          ; 46: Motor Control PWM
QEI_IRQHandler            ; 47: Quadrature Encoder Interface
PLL1_IRQHandler           ; 48: PLL1 Lock (USB PLL)
USBActivity_IRQHandler    ; 49: USB Activity interrupt to wakeup
CANActivity_IRQHandler    ; 50: CAN Activity interrupt to wakeup
UART4_IRQHandler          ; 51: UART4
SSP2_IRQHandler           ; 52: SSP2
LCD_IRQHandler            ; 53: LCD
GPIO_IRQHandler           ; 54: GPIO
PWM0_IRQHandler           ; 55: PWM0
EEPROM_IRQHandler         ; 56: EEPROM
CMP0_IRQHandler           ; 57: Komparator_0 ?
CMP1_IRQHandler           ; 58: Komparator_1 ?
       LDR     R1, =istWeakInt
       B       Restart
       ENDP

       ALIGN
       LTORG
       END
