/* EVB4088: setup.c */

#include "StdTypes.h"
#include "LPC4088.h"
#include "startup.h"
#include "config.h"

/*----------- Pin-Funktionen ----------------*/
/* I/O wird bei GPIO Pins per DIR Register bestimmt */
#define PullDn    (1<<3)
#define PullUp    (2<<3)
#define Keeper    (3<<3)
#define Hysterese (1<<5)
#define InvPol    (1<<6)
#define Fast      (1<<9)
#define OpDrain   (1<<10)

#define nix (PullUp+Hysterese)

/*----------- Pin-Belegung ------------------*/
static const word Port0_ConfigVals[32] =
{ 2,                /* P00: U3TxD  */
  2,                /* P01: U3RxD  */
  1,                /* P02: U0TxD  */
  1,                /* P03: U0RxD  */
  1+Fast,           /* P04: Audio RxClk  */
  1+Fast,           /* P05: Audio RxWS   */
  1+Fast,           /* P06: Audio RxDat  */
  nix,              /* P07: (Audio TxClk)  */
  nix,              /* P08: (Audio TxWS)   */
  nix,              /* P09: (Audio TxDat)  */
  1,                /* P10: U2TxD  */
  1,                /* P11: U2RxD  */
  PullUp+Hysterese, /* P12: CDI  (sw-->WPI-->gnd) */
  1,                /* P13: USB UP LED2   */
  3,                /* P14: USB Connect2  */
  2,                /* P15: SSP0 SCK  */
  2,                /* P16: SSP0 SSEL */
  2,                /* P17: SSP0 MISO */
  2,                /* P18: SSP0 MOSI */
  nix,              /* P19: Kbd  CLK  */
  nix,              /* P20: Kbd  DAT  */
  nix,              /* P21: Maus CLK  */
  nix,              /* P22: Maus DAT  */
  1,                /* P23: ADC0      */
  3,                /* P24: T3_CAP1   */
  1,                /* P25: ADC2      */
  2,                /* P26: DAC Out   */
  1,                /* P27: I2C0 SDA  */
  1,                /* P28: I2C0 SCL  */
  1+Fast,           /* P29: USB1 D+  (Host) */
  1+Fast,           /* P30: USB1 D-  (Host) */
  1+Fast,           /* P31: USB2 D+  (Device) */
};

static const word Port1_ConfigVals[32] =
{ nix,              /* P00: (Ethernet TxD0)  */
  nix,              /* P01: (Ethernet TxD1)  */
  2+Fast,           /* P02: SDIO CLK  */
  2+Fast,           /* P03: SDIO CMD  */
  nix,              /* P04: (Ethernet TxEN)  */
  2,                /* P05: SDIO PWR  */
  2+Fast,           /* P06: SDIO DAT0 */
  2+Fast,           /* P07: SDIO DAT1 */
  nix,              /* P08: (Ethernet CRS DV) */
  nix,              /* P09: (Ethernet RxD0)   */
  nix,              /* P10: (Ethernet RxD0)   */
  2+Fast,           /* P11: SDIO DAT2 */
  2+Fast,           /* P12: SDIO DAT3 */
  nix,              /* P13:   */
  nix,              /* P14: (Ethernet RX ER)  */
  nix,              /* P15: (Ethernet RefClk) */
  nix,              /* P16: (Ethernet MDC)    */
  nix,              /* P17: (Ethernet MDIO)   */
  5+Fast,           /* P18: SSP1 MISO  */
  5+Fast,           /* P19: SSP1 SCK   */
  7+Fast,           /* P20: LCD  VD10 */
  7+Fast,           /* P21: LCD  VD11 */
  7+Fast,           /* P22: LCD  VD12 */
  7+Fast,           /* P23: LCD  VD13 */
  7+Fast,           /* P24: LCD  VD14 */
  7+Fast,           /* P25: LCD  VD15 */
  7+Fast,           /* P26: LCD  VD20 */
  7+Fast,           /* P27: LCD  VD21 */
  7+Fast,           /* P28: LCD  VD22 */
  7+Fast,           /* P29: LCD  VD23 */
  2,                /* P30: USB VBUS  */
  PullUp+Hysterese, /* P31: WPI (sw-->gnd)  */
};

static const word Port2_ConfigVals[32] =
{ 2,                /* P00: U2 TxD    */
  2,                /* P01: U2 RxD    */
  7+Fast,           /* P02: LCD DCLK  */
  7+Fast,           /* P03: LCD VSYN  */
  7+Fast,           /* P04: LCD DTMG  */
  7+Fast,           /* P05: LCD HSYN  */
  7+Fast,           /* P06: LCD VD4   */
  7+Fast,           /* P07: LCD VD5   */
  7+Fast,           /* P08: LCD VD6   */
  7+Fast,           /* P09: LCD VD7   */
  nix,              /* P10: Bootlader */
  nix,              /* P11: (LCD Aux Clock In) */
  5+Fast,           /* P12: LCD VD3   */
  7+Fast,           /* P13: LCD VD19  */
  2,                /* P14: I2C1 SDA  */
  2,                /* P15: I2C1 SCL  */
  1+Fast,           /* P16: SDRAM CAS */
  1+Fast,           /* P17: SDRAM RAS */
  1,                /* P18: SDRAM CLK0 */
  nix,              /* P19:   */
  1+Fast,           /* P20: SDRAM DYCS0 */
  nix,              /* P21:   */
  nix,              /* P22:   */
  nix,              /* P23:   */
  1+Fast,           /* P24: SDRAM CKE  */
  nix,              /* P25:   */
  nix,              /* P26:   */
  nix,              /* P27:   */
  1+Fast,           /* P28: SDRAM DQM0 */
  1+Fast,           /* P29: SDRAM DQM1 */
  1+Fast,           /* P30: SDRAM DQM2 */
  1+Fast,           /* P31: SDRAM DQM3 */
};

/* Port 3 ist komplett Datenbus */


static const word Port4_ConfigVals[32] =
{ 0,                /* P00:   */
  0,                /* P01:   */
  0,                /* P02:   */
  0,                /* P03:   */
  0,                /* P04:   */
  0,                /* P05:   */
  0,                /* P06:   */
  0,                /* P07:   */
  0,                /* P08:   */
  0,                /* P09:   */
  0,                /* P10:   */
  0,                /* P11:   */
  0,                /* P12:   */
  0,                /* P13:   */
  0,                /* P14:   */
  0,                /* P15:   */
  0,                /* P16:   */
  0,                /* P17:   */
  0,                /* P18:   */
  0,                /* P19:   */
  0,                /* P20:   */
  0,                /* P21:   */
  0,                /* P22:   */
  0,                /* P23:   */
  0,                /* P24:   */
  0,                /* P25:   */
  0,                /* P26:   */
  0,                /* P27:   */
  0,                /* P28:   */
  0,                /* P29:   */
  0,                /* P30:   */
  0,                /* P31:   */
};

static const word Port5_ConfigVals[5] =
{ 0,                /* P00:   */
  0,                /* P01:   */
  0,                /* P02:   */
  0,                /* P03:   */
  0,                /* P04:   */
};





/*----------- Gerte-Status-Variablen ----------------*/
dword  ExtRamSize;
bool   SD_CardDetected;
bool   SD_readonly;
int64  SD_CardSize;
int    SDCardTyp;     /* siehe sdio.h "Supported Memory Cards" */


void delay (long wielange)
{ long L;
  L = wielange;
  while (--L);
}


/*----------- externer SDRAM --------------------------
IC = Micron MT48LC4M32B2-7
     1 Mega x 32 Bit x 4 Bnke
     Refreshcount      = 4 K
     ROW Adressing     = 4 K (A0..A11) row length = 12
     COL Adressing     = 256 (A0..A7)  col length = 8
     Bank Adressing    = 4 (BA0, BA1)
     max Clockfrequenz =  143 MHz
     Accesstime        =  5.5 ns (CL=3 ; CAS read latency)


konkret angeschlossen ist der SDRAM so:
RAM A0..A11  an LPC  A0..A11
RAM Pin69 (A12) an LPC A12
RAM BA0      an LPC  A13
RAM BA1      an LPC  A14
RAM CKE      an LPC  CKE_0
RAM CLK      an LPC  CLK_0
RAM CS       an LPC  DYCS0
RAM WE       an LPC  WE
RAM CAS      an LPC  CAS
RAM RAS      an LPC  RAS
RAM DQM0     an LPC  DQM_0
RAM DQM1     an LPC  DQM_1
RAM DQM2     an LPC  DQM_2
RAM DQM3     an LPC  DQM_3
RAM DQ0..31  an LPC  DQ0..31

EMC luft mit CCLK (also EMCCLK=CCLK), macht 60 MHz, also 16.7 ns pro Clocktakt

Mapping laut NXP: (B=Busbreite, L=LoPower, M=Mapping Bits)
32 bit external bus high-performance address mapping (Row, Bank, Column)
(MB = MegaBits)
========================================================================
B L MMM MM
1 0 010 10 = 128 MB (4Mx32),  4 banks, row length = 12, column length = 8   <--?
1 1 010 10 = 128 MB (4Mx32),  4 banks, row length = 12, column length = 8   <--loPower

also Mapping = 0x01010
*/

#define Mapping  (0x0A)              // bin = 01010;

/*--- EMC_DYN_CFG0 = Konfiguration -----*/
#define MD    (01 << 3)              // LoPower SDRAM
#define AM    (Mapping << 7)         // je nach SDRAM-Typ
#define B12   (1 << 12)              // 1 = LoPower, 0 = sonst
#define B14   (1 << 14)              // Busbreite
#define BufferEnable  (1 << 19)      // erst nach Init einschalten

#define ValFor_EMC_DYN_CFG0    (MD | AM | B12 | B14)

/*-------- Kommandos zum EMC -----------*/
#define NORMAL_CMD   (0<<7)   // NORMAL Command
#define MODE_CMD     (1<<7)   // MODE Command
#define PALL_CMD     (2<<7)   // Precharge All Command
#define NOP_CMD      (3<<7)   // NOP Command

/*-------- Mode-Register im SDRAM ------*/
/*
Bit 2..0: Burst length
       0 = 1er burst
       1 = 2er burst
       2 = 4er burst   <--- das benutzen, weil 32 Bit
       3 = 8er burst
       4..6 = verboten
       7 = full page bei M3=0
Bit 3: M3   0=sequential, 1=interleaved  ( 0 benutzen )
Bit 6..4: CAS latency  erlaubt: 1, 2 oder 3  ( 2 benutzen )
Bit 9: Write Burst Mode:
       0=programmierte Burstlnge,    <--- das benutzen
       1=single location access

Der Zugriff ist gemein!! nmlich ohne Daten.
Stattdessen Lesezugriff ber SDRAM-Startadresse+Wert fr Moderegister
also read [0xA0000000 + (WertFrModereg shl 10)]
*/

#define MD_BurstLen     (2 << 0)  // 4er Burst
#define MD_Interleaved  (0 << 3)  // 1 = ein, 0 = aus = sequenziell
#define MD_CAS_Latenz   (2 << 4)  // soll zu EMC_DYN_RASCAS0 passen

#define MD_word     (MD_BurstLen | MD_Interleaved | MD_CAS_Latenz)
#define modesetadr  (0xA0000000 | (MD_word << 10))    // 2 Bit wegen Datenbreite, 8 Bit CAS




volatile dword  D;




long sysconfig (void)
{ dword* PDW;
  int    i;
  long   L;


  i = 0;
  while (i<32)
  { IOCON_P0[i] = Port0_ConfigVals[i];
    ++i;
  }

  i = 0;
  while (i<32)
  { IOCON_P1[i] = Port1_ConfigVals[i];
    ++i;
  }

  i = 0;
  while (i<32)
  { IOCON_P2[i] = Port2_ConfigVals[i];
    ++i;
  }

  i = 0;
  while (i<32)
  { IOCON_P3[i] = 1+Fast;
    ++i;
  }

  i = 0;
  while (i<32)
  { IOCON_P4[i] = Port4_ConfigVals[i];
    ++i;
  }

  i = 0;
  while (i<5)
  { IOCON_P5[i] = Port5_ConfigVals[i];
    ++i;
  }


/*----------- Pin I/O setzen -------------------------*/
  P1_DIR |= 0x80000000;         // WPI (SD-karte) auf 0
  P1_CLR  = 0x80000000;



  PCONP |= (1<<20);  // Power fr LCD einschalten
  LCD_CTRL = 0;      // erstmal alles aus

//  ExtRamBegin     = 0xA0000000;
  ExtRamSize      = 0;

/*----------- externer SDRAM --------------------------*/
  PCONP          |= (1<<11);  // Power fr EMC einschalten
  EMC_CONTROL     = 1;        // EMC enable, Adressmirror aus, normal mode
  EMC_CONFIG      = 0;        //(0) Little Endian mode
  DYNAMICCONTROL  = 2;        //
  DYNAMICRP       = 1;        //(1) tRP    (mind. 20ns) auf (n+1) = 2 Takte = 33 ns
  DYNAMICRAS      = 4;        //(3) tRAS   (mind. 42ns) auf (n+1) = 4 Takte = 66 ns
  DYNAMICSREX     = 5;        // tSREX=tXSR (mind. 70ns) auf (n+1) = 6 Takte = 100 ns
  DYNAMICAPR      = 1;        //(1) tAPR in Clocks (nicht gefunden)
  DYNAMICDAL      = 5;        // tDAL (CL=3) --> 5 Takte
  DYNAMICWR       = 1;        //(1) tWR (1 CLK+7ns) also (n+1) = 2 Takte
  DYNAMICRC       = 5;        // tRC    (mind. 70ns) auf (n+1) = 6 Takte = 100 ns
  DYNAMICRFC      = 5;        // tRFC   (mind. 70ns) auf (n+1) = 6 Takte = 100 ns
  DYNAMICXSR      = 5;        // tXSR   (mind. 70ns) auf (n+1) = 6 Takte = 100 ns
  DYNAMICRRD      = 1;        // tRRD   (mind. 14ns) auf (n+1) = 2 Takte = 33 ns
  DYNAMICMRD      = 1;        // tMRD = (n+1) = 2 Takte

  DYNAMICREADCONFIG = 1;      // "Command delayed strategie" --> benutzt EMCCLKDELAY
  DYNAMICRASCAS0    = 0x0202; //0202 je 2 Takte
  DYNAMICCONFIG0    = ValFor_EMC_DYN_CFG0;

  delay(1000);

  L = 2000;
  while (--L)
  { delay(1);
    DYNAMICCONTROL = NOP_CMD | 3;      // zuerst NOP Command + alle Takte ein
  }
  delay(20000);
  DYNAMICCONTROL = PALL_CMD | 3;       // Precharge Command + alle Takte ein
  DYNAMICREFRESH = 2;                  // erstmal 2*16 = alle 32 CCLK's Refresh ..?
  delay(20000);                        // und das lange...
  DYNAMICREFRESH = 14;                 // jetzt alle 14*16 CCLK's nen Refresh
  DYNAMICCONTROL = MODE_CMD | 3;       // jetzt kommt's zum Moderegister setzen
  PDW = (dword*)modesetadr;
  D = *PDW;                            // dummy read von berechneter Adresse
  delay(2000);
  DYNAMICCONTROL = NORMAL_CMD;         // endlich zum Normalmode kommen
  DYNAMICCONFIG0 = ValFor_EMC_DYN_CFG0 | BufferEnable; // Puffer einschalten
  delay(20000);                        // eigentlich sollte der SDRAM jetzt funktionieren

  SD_CardDetected = false;
  SD_readonly     = false;
  SD_CardSize     = 0;
  SDCardTyp       = 0;

  return D;                            // damit der Compiler mir das nicht wegoptimiert
}




/* ein billiger RAM Test: Bereich ist eigentlich A0000000 .. AFFFFFFF
   aber ab A1000000 friert der uC ein - anstelle Data Abort.
 */
void ExtRamTest(void)
{ dword* PDW;
  dword* PDW0;
  dword* PDEND;

  PDEND = (dword*) CgBWSP;

  PDW0 = PDW = (dword*) ExtRamBegin;
  *PDW0 = 0x4711AFFE;
  ++PDW;

  while ((PDW < PDEND) && (*PDW0 == 0x4711AFFE))
  { *PDW = 0xFFFEFFFE;
    ++PDW;
  }

  *PDW0 = 0xFFFEFFFE;
  while (PDW0 < PDW)
  { if (*PDW0 == 0xFFFEFFFE) *PDW0 = *PDW0 + 0x10002;
    ++PDW0;
  }

  PDW0 = (dword*)ExtRamBegin;
  while ((PDW0 < PDW) && (*PDW0==0)) ++PDW0;
  ExtRamSize = (dword)PDW0 - (dword)ExtRamBegin;
}


/*  ende */
