/***** LPC4088 (LPC4088.h) *****/

#ifndef LPC408XINCLUDED
#define LPC408XINCLUDED


#ifndef dword
#define dword unsigned long
#endif

#ifndef word
#define word unsigned short
#endif

#ifndef byte
#define byte unsigned char
#endif

/************ CORTEX M4 CPU-interne Register lt. ARM ************************/
/* Copyright  2009, 2010 ARM Limited. All rights reserved. */
#define ACTLR    (*((volatile dword *) 0xE000E008)) // RW 0x00000000 Auxiliary Control Register, ACTLR on page 4-5
#define STCSR    (*((volatile dword *) 0xE000E010)) // RW 0x00000000 SysTick Control and Status Register
#define STCTRL   STCSR   // redef by NXP
#define STRVR    (*((volatile dword *) 0xE000E014)) // RW Unknown    SysTick Reload Value Register
#define STRELOAD STRVR   // redef by NXP
#define STCVR    (*((volatile dword *) 0xE000E018)) // RW clear Unknown SysTick Current Value Register 
#define STCURR   STCVR   // redef by NXP
#define STCR     (*((volatile dword *) 0xE000E01C)) // RO STCALIB SysTick Calibration Value Register 
#define STCALIB  STCR    // redef by NXP
#define CPUID    (*((volatile dword *) 0xE000ED00)) // RO 0x410FC241 CPUID Base Register, CPUID on page 4-5
#define ICSR     (*((volatile dword *) 0xE000ED04)) // RW or RO 0x00000000 Interrupt Control and State Register
#define VTOR     (*((volatile dword *) 0xE000ED08)) // RW 0x00000000 Vector Table Offset Register
#define AIRCR    (*((volatile dword *) 0xE000ED0C)) // RW 0x00000000 a Application Interrupt and Reset Control Register
#define SCR_C4   (*((volatile dword *) 0xE000ED10)) // RW 0x00000000 System Control Register
#define CCR_C4   (*((volatile dword *) 0xE000ED14)) // RW 0x00000200 Configuration and Control Register.
#define SHPR1    (*((volatile dword *) 0xE000ED18)) // RW 0x00000000 System Handler Priority Register 1
#define SHPR2    (*((volatile dword *) 0xE000ED1C)) // RW 0x00000000 System Handler Priority Register 2
#define SHPR3    (*((volatile dword *) 0xE000ED20)) // RW 0x00000000 System Handler Priority Register 3
#define SHCSR    (*((volatile dword *) 0xE000ED24)) // RW 0x00000000 System Handler Control and State Register
#define CFSR     (*((volatile dword *) 0xE000ED28)) // RW 0x00000000 Configurable Fault Status Registers
#define HFSR     (*((volatile dword *) 0xE000ED2C)) // RW 0x00000000 HardFault Status Register
#define DFSR     (*((volatile dword *) 0xE000ED30)) // RW 0x00000000 Debug Fault Status Register
#define MMFAR    (*((volatile dword *) 0xE000ED34)) // RW Unknown MemManage Fault Address Registerb
#define BFAR     (*((volatile dword *) 0xE000ED38)) // RW Unknown BusFault Address Registerb
#define AFSR     (*((volatile dword *) 0xE000ED3C)) // RW 0x00000000 Auxiliary Fault Status Register, AFSR on page 4-6
#define ID_PFR0  (*((volatile dword *) 0xE000ED40)) // RO 0x00000030 Processor Feature Register 0
#define ID_PFR1  (*((volatile dword *) 0xE000ED44)) // RO 0x00000200 Processor Feature Register 1
#define ID_DFR0  (*((volatile dword *) 0xE000ED48)) // RO 0x00100000 Debug Features Register 0c
#define ID_AFR0  (*((volatile dword *) 0xE000ED4C)) // RO 0x00000000 Auxiliary Features Register 0
#define ID_MMFR0 (*((volatile dword *) 0xE000ED50)) // RO 0x00100030 Memory Model Feature Register 0
#define ID_MMFR1 (*((volatile dword *) 0xE000ED54)) // RO 0x00000000 Memory Model Feature Register 1
#define ID_MMFR2 (*((volatile dword *) 0xE000ED58)) // RO 0x01000000 Memory Model Feature Register 2
#define ID_MMFR3 (*((volatile dword *) 0xE000ED5C)) // RO 0x00000000 Memory Model Feature Register 3
#define ID_ISAR0 (*((volatile dword *) 0xE000ED60)) // RO 0x01141110 Instruction Set Attributes Register 
#define ID_ISAR1 (*((volatile dword *) 0xE000ED64)) // RO 0x02112000 Instruction Set Attributes Register 1
#define ID_ISAR2 (*((volatile dword *) 0xE000ED68)) // RO 0x21232231 Instruction Set Attributes Register 2
#define ID_ISAR3 (*((volatile dword *) 0xE000ED6C)) // RO 0x01111131 Instruction Set Attributes Register 3
#define ID_ISAR4 (*((volatile dword *) 0xE000ED70)) // RO 0x01310132 Instruction Set Attributes Register 4
#define CPACR    (*((volatile dword *) 0xE000ED88)) // RW - Coprocessor Access Control Register
#define STIR     (*((volatile dword *) 0xE000EF00)) // WO 0x00000000 Software Triggered Interrupt Register


/************ CORTEX M4 MPU ***************************/
#define MPU_TYPE    (*((volatile dword *) 0xE000ED90)) // RO 0x00000800 MPU Type Register (0 if not present)
#define MPU_CTRL    (*((volatile dword *) 0xE000ED94)) // RW 0x00000000 MPU Control Register
#define MPU_RNR     (*((volatile dword *) 0xE000ED98)) // RW 0x00000000 MPU Region Number Register
#define MPU_RBAR    (*((volatile dword *) 0xE000ED9C)) // RW 0x00000000 MPU Region Base Address Register
#define MPU_RASR    (*((volatile dword *) 0xE000EDA0)) // RW 0x00000000 MPU Region Attribute and Size Register
#define MPU_RBAR_A1 (*((volatile dword *) 0xE000EDA4)) //    0x00000000 MPU alias registers
#define MPU_RASR_A1 (*((volatile dword *) 0xE000EDA8)) //    0x00000000
#define MPU_RBAR_A2 (*((volatile dword *) 0xE000EDAC)) //    0x00000000
#define MPU_RASR_A2 (*((volatile dword *) 0xE000EDB0)) //    0x00000000
#define MPU_RBAR_A3 (*((volatile dword *) 0xE000EDB4)) //    0x00000000
#define MPU_RASR_A3 (*((volatile dword *) 0xE000EDB8)) //    0x00000000



/************ CORTEX M4 NVIC Register *****************/
#define ICTR        (*((volatile dword *) 0xE000E004))  // RO - Interrupt Controller Type Register
#define NVIC_ISER0  (*((volatile dword *) 0xE000E100 )) // RW
#define NVIC_ISER1  (*((volatile dword *) 0xE000E104 )) // RW

#define NVIC_ICER0  (*((volatile dword *) 0xE000E180 )) // RW
#define NVIC_ICER1  (*((volatile dword *) 0xE000E184 )) // RW

#define NVIC_ISPR0  (*((volatile dword *) 0xE000E200 )) // RW
#define NVIC_ISPR1  (*((volatile dword *) 0xE000E204 )) // RW

#define NVIC_ICPR0  (*((volatile dword *) 0xE000E280 )) // RW
#define NVIC_ICPR1  (*((volatile dword *) 0xE000E284 )) // RW

#define NVIC_IABR0  (*((volatile dword *) 0xE000E300 )) // RO
#define NVIC_IABR1  (*((volatile dword *) 0xE000E304 )) // RO

/* Prioritten, Resetwert: alles auf 0. 0=hchste, 1F=geringste */
#define NVIC_IPR0   (*((volatile dword *) 0xE000E400 )) // RW
#define NVIC_IPR1   (*((volatile dword *) 0xE000E404 )) // RW
#define NVIC_IPR2   (*((volatile dword *) 0xE000E408 )) // RW
#define NVIC_IPR3   (*((volatile dword *) 0xE000E40C )) // RW
#define NVIC_IPR4   (*((volatile dword *) 0xE000E410 )) // RW
#define NVIC_IPR5   (*((volatile dword *) 0xE000E414 )) // RW
#define NVIC_IPR6   (*((volatile dword *) 0xE000E418 )) // RW
#define NVIC_IPR7   (*((volatile dword *) 0xE000E41C )) // RW
#define NVIC_IPR8   (*((volatile dword *) 0xE000E420 )) // RW
#define NVIC_IPR9   (*((volatile dword *) 0xE000E424 )) // RW
#define NVIC_IPR10  (*((volatile dword *) 0xE000E428 )) // RW

#define NVIC_STIR   (*((volatile dword *) 0xE000EF00 )) // WO

typedef enum IRQn
{
/******  Cortex-M4 Processor Exceptions Numbers *************************************************/
//................................ Nr. .... Prioritt
  Reset_IRQn                    = -15,      // 1  Reset Vector, on PowerUp and warm reset       
  NonMaskableInt_IRQn           = -14,      // 2  Non Maskable Interrupt                        
  HardFault_IRQn                = -13,      // 3  Hard Fault, all classes of Fault              
  MemoryManagement_IRQn         = -12,      // 4  Cortex-M3 Memory Management Interrupt         
  BusFault_IRQn                 = -11,      // 5  Cortex-M3 Bus Fault Interrupt                 
  UsageFault_IRQn               = -10,      // 6  Cortex-M3 Usage Fault Interrupt               
  SVCall_IRQn                   = -5,       // 11 Cortex-M3 SV Call Interrupt                   
  DebugMonitor_IRQn             = -4,       // 12 Cortex-M3 Debug Monitor Interrupt             
  PendSV_IRQn                   = -2,       // 14 Cortex-M3 Pend SV Interrupt                   
  SysTick_IRQn                  = -1,       // 15 Cortex-M3 System Tick Interrupt               

/******  LPC4088 Specific Interrupt Numbers *****************************************************/
  WDT_IRQn                      = 0,        // 16 Watchdog Timer Interrupt                      
  TIMER0_IRQn                   = 1,        // 17 Timer0 Interrupt                              
  TIMER1_IRQn                   = 2,        // 18 Timer1 Interrupt                              
  TIMER2_IRQn                   = 3,        // 19 Timer2 Interrupt                              
  TIMER3_IRQn                   = 4,        // 20 Timer3 Interrupt                              
  UART0_IRQn                    = 5,        // 21 UART0 Interrupt                               
  UART1_IRQn                    = 6,        // 22 UART1 Interrupt                               
  UART2_IRQn                    = 7,        // 23 UART2 Interrupt                               
  UART3_IRQn                    = 8,        // 24 UART3 Interrupt                               
  PWM1_IRQn                     = 9,        // 25 PWM1 Interrupt                                
  I2C0_IRQn                     = 10,       // 26 I2C0 Interrupt                                
  I2C1_IRQn                     = 11,       // 27 I2C1 Interrupt                                
  I2C2_IRQn                     = 12,       // 28 I2C2 Interrupt                                
  unused /*SPI_IRQn*/           = 13,       // 29 SPI Interrupt                                 
  SSP0_IRQn                     = 14,       // 30 SSP0 Interrupt                                
  SSP1_IRQn                     = 15,       // 31 SSP1 Interrupt                                
  PLL0_IRQn                     = 16,       // 32 PLL0 Lock (Main PLL) Interrupt                
  RTC_IRQn                      = 17,       // 33 Real Time Clock Interrupt                     
  EINT0_IRQn                    = 18,       // 34 External Interrupt 0 Interrupt                
  EINT1_IRQn                    = 19,       // 35 External Interrupt 1 Interrupt                
  EINT2_IRQn                    = 20,       // 36 External Interrupt 2 Interrupt                
  EINT3_IRQn                    = 21,       // 37 External Interrupt 3 Interrupt                
  ADC_IRQn                      = 22,       // 38 A/D Converter Interrupt                       
  BOD_IRQn                      = 23,       // 39 Brown-Out Detect Interrupt                    
  USB_IRQn                      = 24,       // 40 USB Interrupt                                 
  CAN_IRQn                      = 25,       // 41 CAN Interrupt                                 
  DMA_IRQn                      = 26,       // 42 General Purpose DMA Interrupt                 
  I2S_IRQn                      = 27,       // 43 I2S Interrupt                                 
  ENET_IRQn                     = 28,       // 44 Ethernet Interrupt                            
  SDIO_IRQn                     = 29,       // 45 SD Card Interrupt                             
  MCPWM_IRQn                    = 30,       // 46 Motor Control PWM Interrupt                   
  QEI_IRQn                      = 31,       // 47 Quadrature Encoder Interface Interrupt        
  PLL1_IRQn                     = 32,       // 48 PLL1 Lock (USB PLL) Interrupt                  
  USBActivity_IRQn              = 33,       // 49 USB Activity Interrupt 			
  CANActivity_IRQn              = 34,       // 50 CAN Activity Interrupt 			
  UART4_IRQn                    = 35,       // 51 UART4 Interrupt                               
  SSP2_IRQn                     = 36,       // 52 SSP2 Interrupt                                
  LCD_IRQn                      = 37,       // 53 LCD Interrupt                                
  GPIO_IRQn                     = 38,       // 54 GPIO Interrupt P0xREI, P2xREI, P0xFEI, P2xFEI
  PWM0_IRQn                     = 39,       // 55 PWM0: Match 0 - 6, Capture 0-1                                
  EEPROM_IRQn                   = 40,       // 56 EEPROM: EE_PROG_DONE, EE_RW_DONE
} IRQn_Type;





/************* LPC4088 System and Clock control registers ************************/
#define MATRIXARB   (*((volatile dword *) 0x400FC188)) // Matrix Arbitration register

/* Phase Locked Loop (PLL0, Main PLL) */
#define PLL0CON     (*((volatile dword *) 0x400FC080)) // PLL0 Control Register R/W 0
#define PLL0CFG     (*((volatile dword *) 0x400FC084)) // PLL0 Configuration Register R/W 0
#define PLL0STAT    (*((volatile dword *) 0x400FC088)) // PLL0 Status Register RO 0
#define PLL0FEED    (*((volatile dword *) 0x400FC08C)) // PLL0 Feed Register WO NA

/* Phase Locked Loop (PLL1, USB PLL) */
#define PLL1CON     (*((volatile dword *) 0x400FC0A0)) // PLL1 Control Register R/W 0
#define PLL1CFG     (*((volatile dword *) 0x400FC0A4)) // PLL1 Configuration Register R/W 0
#define PLL1STAT    (*((volatile dword *) 0x400FC0A8)) // PLL1 Status Register RO 0
#define PLL1FEED    (*((volatile dword *) 0x400FC0AC)) // PLL1 Feed Register WO NA

/* Power control */
#define PCON        (*((volatile dword *) 0x400FC0C0)) // Power Control Register R/W 0
#define PCONP       (*((volatile dword *) 0x400FC0C4)) // Power Control for Peripherals Register0 R/W 0x0408829E
#define PCONP1      (*((volatile dword *) 0x400FC0C8)) // Power Control for Peripherals Register1 R/W 0x8
#define PBOOST      (*((volatile dword *) 0x400FC1B0)) // Power Boost Register R/W 0x3

/* Clock selection and divider registers */
#define EMCCLKSEL   (*((volatile dword *) 0x400FC100)) // External Memory Controller Clock Selection register R/W 0
#define CCLKSEL     (*((volatile dword *) 0x400FC104)) // CPU Clock Configuration Register R/W 0
#define USBCLKSEL   (*((volatile dword *) 0x400FC108)) // USB Clock Configuration Register R/W 0
#define CLKSRCSEL   (*((volatile dword *) 0x400FC10C)) // Clock Source Select Register R/W 0
#define PCLKSEL     (*((volatile dword *) 0x400FC1A8)) // Peripheral Clock Selection register  R/W 0
#define SPIFICLKSEL (*((volatile dword *) 0x400FC1AC)) // SPIFI Clock Selection register  R/W 0

/* External interrupts */
#define EXTINT      (*((volatile dword *) 0x400FC140)) // External Interrupt Flag Register R/W
#define EXTMODE     (*((volatile dword *) 0x400FC148)) // External Interrupt Mode register R/W
#define EXTPOLAR    (*((volatile dword *) 0x400FC14C)) // External Interrupt Polarity Register R/W

/* Device and peripheral reset */
#define RSID        (*((volatile dword *) 0x400FC180)) // Reset Source Identification Register R/W
#define RSTCON0     (*((volatile dword *) 0x400FC1CC)) // Individual peripheral reset control bits R/W
#define RSTCON1     (*((volatile dword *) 0x400FC1D0)) // Individual peripheral reset control bits R/W

/* EMC delay control and calibration */
#define EMCDLYCTL    (*((volatile dword *) 0x400FC1DC)) // values associated with SDRAM operation. R/W 0x210 
#define EMCCAL       (*((volatile dword *) 0x400FC1E0)) // calibration counter R/W 0x1F00 

/* Miscellaneous system control registers */
#define SCS          (*((volatile dword *) 0x400FC1A0)) // System Control and Status
#define IRCCTRL      (*((volatile dword *) 0x400FC1A4)) // IRC control R/W 0x80 
#define LCD_CFG      (*((volatile dword *) 0x400FC1B8)) // LCD Clock configuration register 
#define CANSLEEPCLR  (*((volatile dword *) 0x400FC110)) // current CAN channel sleep state 
#define CANWAKEFLAGS (*((volatile dword *) 0x400FC114)) // Indicates the wake-up state of the CAN channels
#define USBINTST     (*((volatile dword *) 0x400FC1C0)) // USB Interrupt Status R/W 0x80000000 
#define DMACREQSEL   (*((volatile dword *) 0x400FC1C4)) // alternative requests on DMA channels 
#define CLKOUTCFG    (*((volatile dword *) 0x400FC1C8)) // Clock Output Configuration register


/************ LPC4088 Flash Accelerator **************************************/
#define FLASHCFG  (*((volatile dword *) 0x400FC000))


/************ LPC4088 I/O Configuration **************************************/
/* ber Startupcode als arrays definiert */
extern volatile dword IOCON_P0[32];
extern volatile dword IOCON_P1[32];
extern volatile dword IOCON_P2[32];
extern volatile dword IOCON_P3[32];
extern volatile dword IOCON_P4[32];
extern volatile dword IOCON_P5[5];

#define IOCON_P0_0  (*((volatile dword *) 0x4002C000))  // Typ D
#define IOCON_P0_1  (*((volatile dword *) 0x4002C004))  // Typ D
#define IOCON_P0_2  (*((volatile dword *) 0x4002C008))  // Typ D
#define IOCON_P0_3  (*((volatile dword *) 0x4002C00C))  // Typ D
#define IOCON_P0_4  (*((volatile dword *) 0x4002C010))  // Typ D
#define IOCON_P0_5  (*((volatile dword *) 0x4002C014))  // Typ D
#define IOCON_P0_6  (*((volatile dword *) 0x4002C018))  // Typ D
#define IOCON_P0_7  (*((volatile dword *) 0x4002C01C))  // Typ W
#define IOCON_P0_8  (*((volatile dword *) 0x4002C020))  // Typ W
#define IOCON_P0_9  (*((volatile dword *) 0x4002C024))  // Typ W
#define IOCON_P0_10 (*((volatile dword *) 0x4002C028))  // Typ D
#define IOCON_P0_11 (*((volatile dword *) 0x4002C02C))  // Typ D
#define IOCON_P0_12 (*((volatile dword *) 0x4002C030))  // Typ D
#define IOCON_P0_13 (*((volatile dword *) 0x4002C034))  // Typ A
#define IOCON_P0_14 (*((volatile dword *) 0x4002C038))  // Typ A
#define IOCON_P0_15 (*((volatile dword *) 0x4002C03C))  // Typ D
#define IOCON_P0_16 (*((volatile dword *) 0x4002C040))  // Typ D
#define IOCON_P0_17 (*((volatile dword *) 0x4002C044))  // Typ D
#define IOCON_P0_18 (*((volatile dword *) 0x4002C048))  // Typ D
#define IOCON_P0_19 (*((volatile dword *) 0x4002C04C))  // Typ D
#define IOCON_P0_20 (*((volatile dword *) 0x4002C050))  // Typ D
#define IOCON_P0_21 (*((volatile dword *) 0x4002C054))  // Typ D
#define IOCON_P0_22 (*((volatile dword *) 0x4002C058))  // Typ D
#define IOCON_P0_23 (*((volatile dword *) 0x4002C05C))  // Typ A
#define IOCON_P0_24 (*((volatile dword *) 0x4002C060))  // Typ A
#define IOCON_P0_25 (*((volatile dword *) 0x4002C064))  // Typ A 
#define IOCON_P0_26 (*((volatile dword *) 0x4002C068))  // Typ A 
#define IOCON_P0_27 (*((volatile dword *) 0x4002C06C))  // Typ I
#define IOCON_P0_28 (*((volatile dword *) 0x4002C070))  // Typ I
#define IOCON_P0_29 (*((volatile dword *) 0x4002C074))  // Typ U
#define IOCON_P0_30 (*((volatile dword *) 0x4002C078))  // Typ U
#define IOCON_P0_31 (*((volatile dword *) 0x4002C07C))  // Typ U

#define IOCON_P1_0  (*((volatile dword *) 0x4002C080))  // Typ D
#define IOCON_P1_1  (*((volatile dword *) 0x4002C084))  // Typ D
#define IOCON_P1_2  (*((volatile dword *) 0x4002C088))  // Typ D
#define IOCON_P1_3  (*((volatile dword *) 0x4002C08C))  // Typ D
#define IOCON_P1_4  (*((volatile dword *) 0x4002C090))  // Typ D
#define IOCON_P1_5  (*((volatile dword *) 0x4002C094))  // Typ W 
#define IOCON_P1_6  (*((volatile dword *) 0x4002C098))  // Typ W 
#define IOCON_P1_7  (*((volatile dword *) 0x4002C09C))  // Typ W 
#define IOCON_P1_8  (*((volatile dword *) 0x4002C0A0))  // Typ D 
#define IOCON_P1_9  (*((volatile dword *) 0x4002C0A4))  // Typ D 
#define IOCON_P1_10 (*((volatile dword *) 0x4002C0A8))  // Typ D 
#define IOCON_P1_11 (*((volatile dword *) 0x4002C0AC))  // Typ D 
#define IOCON_P1_12 (*((volatile dword *) 0x4002C0B0))  // Typ D 
#define IOCON_P1_13 (*((volatile dword *) 0x4002C0B4))  // Typ D 
#define IOCON_P1_14 (*((volatile dword *) 0x4002C0B8))  // Typ W 
#define IOCON_P1_15 (*((volatile dword *) 0x4002C0BC))  // Typ D 
#define IOCON_P1_16 (*((volatile dword *) 0x4002C0C0))  // Typ W 
#define IOCON_P1_17 (*((volatile dword *) 0x4002C0C4))  // Typ W 
#define IOCON_P1_18 (*((volatile dword *) 0x4002C0C8))  // Typ D
#define IOCON_P1_19 (*((volatile dword *) 0x4002C0CC))  // Typ D 
#define IOCON_P1_20 (*((volatile dword *) 0x4002C0D0))  // Typ D 
#define IOCON_P1_21 (*((volatile dword *) 0x4002C0D4))  // Typ D 
#define IOCON_P1_22 (*((volatile dword *) 0x4002C0D8))  // Typ D 
#define IOCON_P1_23 (*((volatile dword *) 0x4002C0DC))  // Typ D 
#define IOCON_P1_24 (*((volatile dword *) 0x4002C0E0))  // Typ D 
#define IOCON_P1_25 (*((volatile dword *) 0x4002C0E4))  // Typ D 
#define IOCON_P1_26 (*((volatile dword *) 0x4002C0E8))  // Typ D 
#define IOCON_P1_27 (*((volatile dword *) 0x4002C0EC))  // Typ D 
#define IOCON_P1_28 (*((volatile dword *) 0x4002C0F0))  // Typ D 
#define IOCON_P1_29 (*((volatile dword *) 0x4002C0F4))  // Typ D
#define IOCON_P1_30 (*((volatile dword *) 0x4002C0F8))  // Typ A
#define IOCON_P1_31 (*((volatile dword *) 0x4002C0FC))  // Typ A 

#define IOCON_P2_0  (*((volatile dword *) 0x4002C100))  // Typ D 
#define IOCON_P2_1  (*((volatile dword *) 0x4002C104))  // Typ D 
#define IOCON_P2_2  (*((volatile dword *) 0x4002C108))  // Typ D 
#define IOCON_P2_3  (*((volatile dword *) 0x4002C10C))  // Typ D 
#define IOCON_P2_4  (*((volatile dword *) 0x4002C110))  // Typ D
#define IOCON_P2_5  (*((volatile dword *) 0x4002C114))  // Typ D 
#define IOCON_P2_6  (*((volatile dword *) 0x4002C118))  // Typ D
#define IOCON_P2_7  (*((volatile dword *) 0x4002C11C))  // Typ D 
#define IOCON_P2_8  (*((volatile dword *) 0x4002C120))  // Typ D 
#define IOCON_P2_9  (*((volatile dword *) 0x4002C124))  // Typ D 
#define IOCON_P2_10 (*((volatile dword *) 0x4002C128))  // Typ D 
#define IOCON_P2_11 (*((volatile dword *) 0x4002C12C))  // Typ D 
#define IOCON_P2_12 (*((volatile dword *) 0x4002C130))  // Typ D 
#define IOCON_P2_13 (*((volatile dword *) 0x4002C134))  // Typ D 
#define IOCON_P2_14 (*((volatile dword *) 0x4002C138))  // Typ D 
#define IOCON_P2_15 (*((volatile dword *) 0x4002C13C))  // Typ D 
#define IOCON_P2_16 (*((volatile dword *) 0x4002C140))  // Typ D 
#define IOCON_P2_17 (*((volatile dword *) 0x4002C144))  // Typ D
#define IOCON_P2_18 (*((volatile dword *) 0x4002C148))  // Typ D
#define IOCON_P2_19 (*((volatile dword *) 0x4002C14C))  // Typ D 
#define IOCON_P2_20 (*((volatile dword *) 0x4002C150))  // Typ D
#define IOCON_P2_21 (*((volatile dword *) 0x4002C154))  // Typ D 
#define IOCON_P2_22 (*((volatile dword *) 0x4002C158))  // Typ D 
#define IOCON_P2_23 (*((volatile dword *) 0x4002C15C))  // Typ D
#define IOCON_P2_24 (*((volatile dword *) 0x4002C160))  // Typ D 
#define IOCON_P2_25 (*((volatile dword *) 0x4002C164))  // Typ D
#define IOCON_P2_26 (*((volatile dword *) 0x4002C168))  // Typ D
#define IOCON_P2_27 (*((volatile dword *) 0x4002C16C))  // Typ D
#define IOCON_P2_28 (*((volatile dword *) 0x4002C170))  // Typ D
#define IOCON_P2_29 (*((volatile dword *) 0x4002C174))  // Typ D
#define IOCON_P2_30 (*((volatile dword *) 0x4002C178))  // Typ D 
#define IOCON_P2_31 (*((volatile dword *) 0x4002C17C))  // Typ D 

#define IOCON_P3_0  (*((volatile dword *) 0x4002C180))  // Typ D
#define IOCON_P3_1  (*((volatile dword *) 0x4002C184))  // Typ D
#define IOCON_P3_2  (*((volatile dword *) 0x4002C188))  // Typ D
#define IOCON_P3_3  (*((volatile dword *) 0x4002C18C))  // Typ D
#define IOCON_P3_4  (*((volatile dword *) 0x4002C190))  // Typ D
#define IOCON_P3_5  (*((volatile dword *) 0x4002C194))  // Typ D
#define IOCON_P3_6  (*((volatile dword *) 0x4002C198))  // Typ D
#define IOCON_P3_7  (*((volatile dword *) 0x4002C19C))  // Typ D
#define IOCON_P3_8  (*((volatile dword *) 0x4002C1A0))  // Typ D
#define IOCON_P3_9  (*((volatile dword *) 0x4002C1A4))  // Typ D
#define IOCON_P3_10 (*((volatile dword *) 0x4002C1A8))  // Typ D
#define IOCON_P3_11 (*((volatile dword *) 0x4002C1AC))  // Typ D
#define IOCON_P3_12 (*((volatile dword *) 0x4002C1B0))  // Typ D
#define IOCON_P3_13 (*((volatile dword *) 0x4002C1B4))  // Typ D
#define IOCON_P3_14 (*((volatile dword *) 0x4002C1B8))  // Typ D
#define IOCON_P3_15 (*((volatile dword *) 0x4002C1BC))  // Typ D
#define IOCON_P3_16 (*((volatile dword *) 0x4002C1C0))  // Typ D
#define IOCON_P3_17 (*((volatile dword *) 0x4002C1C4))  // Typ D
#define IOCON_P3_18 (*((volatile dword *) 0x4002C1C8))  // Typ D
#define IOCON_P3_19 (*((volatile dword *) 0x4002C1CC))  // Typ D
#define IOCON_P3_20 (*((volatile dword *) 0x4002C1D0))  // Typ D
#define IOCON_P3_21 (*((volatile dword *) 0x4002C1D4))  // Typ D
#define IOCON_P3_22 (*((volatile dword *) 0x4002C1D8))  // Typ D
#define IOCON_P3_23 (*((volatile dword *) 0x4002C1DC))  // Typ D
#define IOCON_P3_24 (*((volatile dword *) 0x4002C1E0))  // Typ D
#define IOCON_P3_25 (*((volatile dword *) 0x4002C1E4))  // Typ D
#define IOCON_P3_26 (*((volatile dword *) 0x4002C1E8))  // Typ D
#define IOCON_P3_27 (*((volatile dword *) 0x4002C1EC))  // Typ D
#define IOCON_P3_28 (*((volatile dword *) 0x4002C1F0))  // Typ D
#define IOCON_P3_29 (*((volatile dword *) 0x4002C1F4))  // Typ D
#define IOCON_P3_30 (*((volatile dword *) 0x4002C1F8))  // Typ D
#define IOCON_P3_31 (*((volatile dword *) 0x4002C1FC))  // Typ D

#define IOCON_P4_0  (*((volatile dword *) 0x4002C200))  // Typ D
#define IOCON_P4_1  (*((volatile dword *) 0x4002C204))  // Typ D
#define IOCON_P4_2  (*((volatile dword *) 0x4002C208))  // Typ D
#define IOCON_P4_3  (*((volatile dword *) 0x4002C20C))  // Typ D
#define IOCON_P4_4  (*((volatile dword *) 0x4002C210))  // Typ D
#define IOCON_P4_5  (*((volatile dword *) 0x4002C214))  // Typ D
#define IOCON_P4_6  (*((volatile dword *) 0x4002C218))  // Typ D
#define IOCON_P4_7  (*((volatile dword *) 0x4002C21C))  // Typ D
#define IOCON_P4_8  (*((volatile dword *) 0x4002C220))  // Typ D
#define IOCON_P4_9  (*((volatile dword *) 0x4002C224))  // Typ D
#define IOCON_P4_10 (*((volatile dword *) 0x4002C228))  // Typ D
#define IOCON_P4_11 (*((volatile dword *) 0x4002C22C))  // Typ D
#define IOCON_P4_12 (*((volatile dword *) 0x4002C230))  // Typ D
#define IOCON_P4_13 (*((volatile dword *) 0x4002C234))  // Typ D
#define IOCON_P4_14 (*((volatile dword *) 0x4002C238))  // Typ D
#define IOCON_P4_15 (*((volatile dword *) 0x4002C23C))  // Typ D
#define IOCON_P4_16 (*((volatile dword *) 0x4002C240))  // Typ D
#define IOCON_P4_17 (*((volatile dword *) 0x4002C244))  // Typ D
#define IOCON_P4_18 (*((volatile dword *) 0x4002C248))  // Typ D
#define IOCON_P4_19 (*((volatile dword *) 0x4002C24C))  // Typ D
#define IOCON_P4_20 (*((volatile dword *) 0x4002C250))  // Typ D
#define IOCON_P4_21 (*((volatile dword *) 0x4002C254))  // Typ D
#define IOCON_P4_22 (*((volatile dword *) 0x4002C258))  // Typ D
#define IOCON_P4_23 (*((volatile dword *) 0x4002C25C))  // Typ D
#define IOCON_P4_24 (*((volatile dword *) 0x4002C260))  // Typ D
#define IOCON_P4_25 (*((volatile dword *) 0x4002C264))  // Typ D
#define IOCON_P4_26 (*((volatile dword *) 0x4002C268))  // Typ D
#define IOCON_P4_27 (*((volatile dword *) 0x4002C26C))  // Typ D
#define IOCON_P4_28 (*((volatile dword *) 0x4002C270))  // Typ D
#define IOCON_P4_29 (*((volatile dword *) 0x4002C274))  // Typ D
#define IOCON_P4_30 (*((volatile dword *) 0x4002C278))  // Typ D
#define IOCON_P4_31 (*((volatile dword *) 0x4002C27C))  // Typ D

#define IOCON_P5_0  (*((volatile dword *) 0x4002C280))  // Typ D 
#define IOCON_P5_1  (*((volatile dword *) 0x4002C284))  // Typ D 
#define IOCON_P5_2  (*((volatile dword *) 0x4002C288))  // Typ I 
#define IOCON_P5_3  (*((volatile dword *) 0x4002C28C))  // Typ I 
#define IOCON_P5_4  (*((volatile dword *) 0x4002C290))  // Typ D 


/************ LPC4088 GPIO Ports 0..4, Base = 0x20098000 *********************/
#define P0_DIR  (*((volatile dword *) 0x20098000))  // R/W GPIO Port0 Direction control register
#define P0_MASK (*((volatile dword *) 0x20098010))  // R/W  Mask register for Port0
#define P0_PIN  (*((volatile dword *) 0x20098014))  // R/W  Port0 Pin value register using FIOMASK
#define P0_SET  (*((volatile dword *) 0x20098018))  // R/W  Port0 Output Set register using FIOMASK
#define P0_CLR  (*((volatile dword *) 0x2009801C))  // WO   Port0 Output Clear register using FIOMASK

#define P1_DIR  (*((volatile dword *) 0x20098020))  // R/W  GPIO Port1 Direction control register
#define P1_MASK (*((volatile dword *) 0x20098030))  // R/W  Mask register for Port1
#define P1_PIN  (*((volatile dword *) 0x20098034))  // R/W  Port1 Pin value register using FIOMASK
#define P1_SET  (*((volatile dword *) 0x20098038))  // R/W  Port1 Output Set register using FIOMASK
#define P1_CLR  (*((volatile dword *) 0x2009803C))  // WO   Port1 Output Clear register using FIOMASK

#define P2_DIR  (*((volatile dword *) 0x20098040))  // R/W  GPIO Port2 Direction control register
#define P2_MASK (*((volatile dword *) 0x20098050))  // R/W  Mask register for Port2
#define P2_PIN  (*((volatile dword *) 0x20098054))  // R/W  Port2 Pin value register using FIOMASK
#define P2_SET  (*((volatile dword *) 0x20098058))  // R/W  Port2 Output Set register using FIOMASK
#define P2_CLR  (*((volatile dword *) 0x2009805C))  // WO   Port2 Output Clear register using FIOMASK

#define P3_DIR  (*((volatile dword *) 0x20098060))  // R/W  GPIO Port3 Direction control register
#define P3_MASK (*((volatile dword *) 0x20098070))  // R/W  Mask register for Port3
#define P3_PIN  (*((volatile dword *) 0x20098074))  // R/W  Port3 Pin value register using FIOMASK
#define P3_SET  (*((volatile dword *) 0x20098078))  // R/W  Port3 Output Set register using FIOMASK
#define P3_CLR  (*((volatile dword *) 0x2009807C))  // WO   Port3 Output Clear register using FIOMASK

#define P4_DIR  (*((volatile dword *) 0x20098080))  // R/W  GPIO Port4 Direction control register
#define P4_MASK (*((volatile dword *) 0x20098090))  // R/W  Mask register for Port4
#define P4_PIN  (*((volatile dword *) 0x20098094))  // R/W  Port4 Pin value register using FIOMASK
#define P4_SET  (*((volatile dword *) 0x20098098))  // R/W  Port4 Output Set register using FIOMASK
#define P4_CLR  (*((volatile dword *) 0x2009809C))  // WO   Port4 Output Clear register using FIOMASK

#define P5_DIR  (*((volatile dword *) 0x200980A0))  // R/W  GPIO Port5 Direction control register
#define P5_MASK (*((volatile dword *) 0x200980B0))  // R/W  Mask register for Port5
#define P5_PIN  (*((volatile dword *) 0x200980B4))  // R/W  Port5 Pin value register using FIOMASK
#define P5_SET  (*((volatile dword *) 0x200980B8))  // R/W  Port5 Output Set register using FIOMASK
#define P5_CLR  (*((volatile dword *) 0x200980BC))  // WO   Port5 Output Clear register using FIOMASK


/************ LPC4088 EMC (external memory controller ***********************/
/* base:  0x2009C000 */
#define EMC_CONTROL        (*((volatile dword *) 0x2009C000))  //  Controls operation of the memory controller
#define EMC_STATUS         (*((volatile dword *) 0x2009C004))  //  RO Provides EMC status information
#define EMC_CONFIG         (*((volatile dword *) 0x2009C008))  //  Configures operation of the memory controller

#define DYNAMICCONTROL     (*((volatile dword *) 0x2009C020))  //  Controls dynamic memory operation
#define DYNAMICREFRESH     (*((volatile dword *) 0x2009C024))  //  Configures dynamic memory refresh
#define DYNAMICREADCONFIG  (*((volatile dword *) 0x2009C028))  //  Configures dynamic memory read strategy
#define DYNAMICRP          (*((volatile dword *) 0x2009C030))  //  Precharge command period
#define DYNAMICRAS         (*((volatile dword *) 0x2009C034))  //  Active to precharge command period
#define DYNAMICSREX        (*((volatile dword *) 0x2009C038))  //  Self-refresh exit time
#define DYNAMICAPR         (*((volatile dword *) 0x2009C03C))  //  Last-data-out to active command time
#define DYNAMICDAL         (*((volatile dword *) 0x2009C040))  //  Data-in to active command time
#define DYNAMICWR          (*((volatile dword *) 0x2009C044))  //  Write recovery time
#define DYNAMICRC          (*((volatile dword *) 0x2009C048))  //  Selects the active to active command period
#define DYNAMICRFC         (*((volatile dword *) 0x2009C04C))  //  Selects the auto-refresh period
#define DYNAMICXSR         (*((volatile dword *) 0x2009C050))  //  Time for exit self-refresh to active command
#define DYNAMICRRD         (*((volatile dword *) 0x2009C054))  //  Latency for active bank A to active bank B
#define DYNAMICMRD         (*((volatile dword *) 0x2009C058))  //  Time for load mode register to active command

#define STATICEXTENDEDWAIT (*((volatile dword *) 0x2009C080))  //  Time for long static memory read and write transfers

#define DYNAMICCONFIG0     (*((volatile dword *) 0x2009C100))  //  Configuration information for EMC_DYCS0
#define DYNAMICRASCAS0     (*((volatile dword *) 0x2009C104))  //  RAS and CAS latencies for EMC_DYCS0
#define DYNAMICCONFIG1     (*((volatile dword *) 0x2009C120))  //  Configuration information for EMC_DYCS1
#define DYNAMICRASCAS1     (*((volatile dword *) 0x2009C124))  //  RAS and CAS latencies for EMC_DYCS1
#define DYNAMICCONFIG2     (*((volatile dword *) 0x2009C140))  //  Configuration information for EMC_DYCS2
#define DYNAMICRASCAS2     (*((volatile dword *) 0x2009C144))  //  RAS and CAS latencies for EMC_DYCS2
#define DYNAMICCONFIG3     (*((volatile dword *) 0x2009C160))  //  Configuration information for EMC_DYCS3
#define DYNAMICRASCAS3     (*((volatile dword *) 0x2009C164))  //  RAS and CAS latencies for EMC_DYCS3

#define STATICCONFIG0      (*((volatile dword *) 0x2009C200))  //  Configuration for EMC_CS0
#define STATICWAITWEN0     (*((volatile dword *) 0x2009C204))  //  Delay from EMC_CS0 to write enable
#define STATICWAITOEN0     (*((volatile dword *) 0x2009C208))  //  Delay from EMC_CS0 or address change to output enable
#define STATICWAITRD0      (*((volatile dword *) 0x2009C20C))  //  Delay from EMC_CS0 to a read access
#define STATICWAITPAGE0    (*((volatile dword *) 0x2009C210))  //  Delay for asynchronous page mode sequential accesses for MC_CS0
#define STATICWAITWR0      (*((volatile dword *) 0x2009C214))  //  Delay from EMC_CS0 to a write access
#define STATICWAITTURN0    (*((volatile dword *) 0x2009C218))  //  Number of bus turnaround cycles EMC_CS0

#define STATICCONFIG1      (*((volatile dword *) 0x2009C220))  //  Memory configuration for EMC_CS1
#define STATICWAITWEN1     (*((volatile dword *) 0x2009C224))  //  Delay from EMC_CS1 to write enable
#define STATICWAITOEN1     (*((volatile dword *) 0x2009C228))  //  Delay from EMC_CS1 or address change to output enable
#define STATICWAITRD1      (*((volatile dword *) 0x2009C22C))  //  Delay from EMC_CS1 to a read access
#define STATICWAITPAGE1    (*((volatile dword *) 0x2009C230))  //  Delay for asynchronous page mode sequential accesses for EMC_CS1
#define STATICWAITWR1      (*((volatile dword *) 0x2009C234))  //  Delay from EMC_CS1 to a write access
#define STATICWAITTURN1    (*((volatile dword *) 0x2009C238))  //  Bus turnaround cycles for EMC_CS1

#define STATICCONFIG2      (*((volatile dword *) 0x2009C240))  //  Memory configuration for EMC_CS2
#define STATICWAITWEN2     (*((volatile dword *) 0x2009C244))  //  Delay from EMC_CS2 to write enable
#define STATICWAITOEN2     (*((volatile dword *) 0x2009C248))  //  Delay from EMC_CS2 or address change to output enable
#define STATICWAITRD2      (*((volatile dword *) 0x2009C24C))  //  Delay from EMC_CS2 to a read access
#define STATICWAITPAGE2    (*((volatile dword *) 0x2009C250))  //  Delay for asynchronous page mode sequential accesses for EMC_CS2
#define STATICWAITWR2      (*((volatile dword *) 0x2009C254))  //  Delay from EMC_CS2 to a write access

#define EMCStaticWaitTurn2 (*((volatile dword *) 0x2009C258))  //  Bus turnaround cycles for EMC_CS2

#define STATICCONFIG3      (*((volatile dword *) 0x2009C260))  //  Memory configuration for EMC_CS3
#define STATICWAITWEN3     (*((volatile dword *) 0x2009C264))  //  Delay from EMC_CS3 to write enable
#define STATICWAITOEN3     (*((volatile dword *) 0x2009C268))  //  Delay from EMC_CS3 or address change to output enable
#define STATICWAITRD3      (*((volatile dword *) 0x2009C26C))  //  Delay from EMC_CS3 to a read access
#define STATICWAITPAGE3    (*((volatile dword *) 0x2009C270))  //  Delay for asynchronous page mode sequential accesses for EMC_CS3
#define STATICWAITWR3      (*((volatile dword *) 0x2009C274))  //  Delay from EMC_CS3 to a write access
#define STATICWAITTURN3    (*((volatile dword *) 0x2009C278))  //  Bus turnaround cycles for EMC_CS3


/************ LPC4088 ETH (Ethernet) ****************************************/
/* base address 0x20084000 */
#define ETH_MAC1  (*((volatile dword *) 0x20084000))  //  MAC configuration register 1
#define ETH_MAC2  (*((volatile dword *) 0x20084004))  //  MAC configuration register 2
#define ETH_IPGT  (*((volatile dword *) 0x20084008))  //  Back-to-Back Inter-Packet-Gap register
#define ETH_IPGR  (*((volatile dword *) 0x2008400C))  //  Non Back-to-Back Inter-Packet-Gap register
#define ETH_CLRT  (*((volatile dword *) 0x20084010))  //  Collision window / Retry register
#define ETH_MAXF  (*((volatile dword *) 0x20084014))  //  Maximum Frame register
#define ETH_SUPP  (*((volatile dword *) 0x20084018))  //  PHY Support register
#define ETH_TEST  (*((volatile dword *) 0x2008401C))  //  Test register
#define ETH_MCFG  (*((volatile dword *) 0x20084020))  //  MII Mgmt Configuration register
#define ETH_MCMD  (*((volatile dword *) 0x20084024))  //  MII Mgmt Command register
#define ETH_MADR  (*((volatile dword *) 0x20084028))  //  MII Mgmt Address register
#define ETH_MWTD  (*((volatile dword *) 0x2008402C))  //  MII Mgmt Write Data register RO
#define ETH_MRDD  (*((volatile dword *) 0x20084030))  //  MII Mgmt Read Data register RO
#define ETH_MIND  (*((volatile dword *) 0x20084034))  //  MII Mgmt Indicators register RO
#define ETH_SA0   (*((volatile dword *) 0x20084040))  //  Station Address 0 register
#define ETH_SA1   (*((volatile dword *) 0x20084044))  //  Station Address 1 register
#define ETH_SA2   (*((volatile dword *) 0x20084048))  //  Station Address 2 register

/* Control registers */
#define ETH_COMMAND            (*((volatile dword *) 0x20084100))  //  Command register
#define ETH_STATUS             (*((volatile dword *) 0x20084104))  //  Status register RO
#define ETH_RXDESCRIPTOR       (*((volatile dword *) 0x20084108))  //  Receive descriptor base address register
#define ETH_RXSTATUS           (*((volatile dword *) 0x2008410C))  //  Receive status base address register
#define ETH_RXDESCRIPTORNUMBER (*((volatile dword *) 0x20084110))  //  Receive number of descriptors register
#define ETH_RXPRODUCEINDEX     (*((volatile dword *) 0x20084114))  //  Receive produce index register RO
#define ETH_RXCONSUMEINDEX     (*((volatile dword *) 0x20084118))  //  Receive consume index register
#define ETH_TXDESCRIPTOR       (*((volatile dword *) 0x2008411C))  //  Transmit descriptor base address register
#define ETH_TXSTATUS           (*((volatile dword *) 0x20084120))  //  Transmit status base address register
#define ETH_TXDESCRIPTORNUMBER (*((volatile dword *) 0x20084124))  //  Transmit number of descriptors register
#define ETH_TXPRODUCEINDEX     (*((volatile dword *) 0x20084128))  //  Transmit produce index register
#define ETH_TXCONSUMEINDEX     (*((volatile dword *) 0x2008412C))  //  Transmit consume index register RO
#define ETH_TSV0               (*((volatile dword *) 0x20084158))  //  Transmit status vector 0 register RO
#define ETH_TSV1               (*((volatile dword *) 0x2008415C))  //  Transmit status vector 1 register RO
#define ETH_RSV                (*((volatile dword *) 0x20084160))  //  Receive status vector register RO
#define ETH_FLOWCONTROLCOUNTER (*((volatile dword *) 0x20084170))  //  Flow control counter register
#define ETH_FLOWCONTROLSTATUS  (*((volatile dword *) 0x20084174))  //  Flow control status register RO

/* Rx filter registers */
#define ETH_RXFILTERCTRL       (*((volatile dword *) 0x20084200))  //  Receive filter control register
#define ETH_RXFILTERWOLSTATUS  (*((volatile dword *) 0x20084204))  //  Receive filter WoL status register RO
#define ETH_RXFILTERWOLCLEAR   (*((volatile dword *) 0x20084208))  //  Receive filter WoL clear register RO
#define ETH_HASHFILTERL        (*((volatile dword *) 0x20084210))  //  Hash filter table LSBs register
#define ETH_HASHFILTERH        (*((volatile dword *) 0x20084214))  //  Hash filter table MSBs register

/* Module control registers */
#define ETH_INTSTATUS          (*((volatile dword *) 0x20084FE0))  //  Interrupt status register RO
#define ETH_INTENABLE          (*((volatile dword *) 0x20084FE4))  //  Interrupt enable register
#define ETH_INTCLEAR           (*((volatile dword *) 0x20084FE8))  //  Interrupt clear register WO
#define ETH_INTSET             (*((volatile dword *) 0x20084FEC))  //  Interrupt set register WO
#define ETH_POWERDOWN          (*((volatile dword *) 0x20084FF4))  //  Power-down register


/************ LPC4088 LCD controller  ****************************************/
/* base address 0x20088000 */
#define LCD_TIMH    (*((volatile dword *) 0x20088000))  //  Horizontal Timing Control register
#define LCD_TIMV    (*((volatile dword *) 0x20088004))  //  Vertical Timing Control register
#define LCD_POL     (*((volatile dword *) 0x20088008))  //  Clock and Signal Polarity Control register
#define LCD_LE      (*((volatile dword *) 0x2008800C))  //  Line End Control register
#define LCD_UPBASE  (*((volatile dword *) 0x20088010))  //  Upper Panel Frame Base Address register
#define LCD_LPBASE  (*((volatile dword *) 0x20088014))  //  Lower Panel Frame Base Address register
#define LCD_CTRL    (*((volatile dword *) 0x20088018))  //  LCD Control register
#define LCD_INTMSK  (*((volatile dword *) 0x2008801C))  //  Interrupt Mask register
#define LCD_INTRAW  (*((volatile dword *) 0x20088020))  //  Raw Interrupt Status register RO
#define LCD_INTSTAT (*((volatile dword *) 0x20088024))  //  Masked Interrupt Status register RO
#define LCD_INTCLR  (*((volatile dword *) 0x20088028))  //  Interrupt Clear register WO
#define LCD_UPCURR  (*((volatile dword *) 0x2008802C))  //  Upper Panel Current Address Value register RO
#define LCD_LPCURR  (*((volatile dword *) 0x20088030))  //  Lower Panel Current Address Value register RO

/* ber Startupcode als arrays definiert */
extern volatile dword LCD_PAL[128];
extern volatile dword LCD_CRSR_IMG[256];

#define LCD_CRSR_CTRL    (*((volatile dword *) 0x20088C00))  //  Cursor Control register
#define LCD_CRSR_CFG     (*((volatile dword *) 0x20088C04))  //  Cursor Configuration register
#define LCD_CRSR_PAL0    (*((volatile dword *) 0x20088C08))  //  Cursor Palette register
#define LCD_CRSR_PAL1    (*((volatile dword *) 0x20088C0C))  //  Cursor Palette register
#define LCD_CRSR_XY      (*((volatile dword *) 0x20088C10))  //  Cursor XY Position register
#define LCD_CRSR_CLIP    (*((volatile dword *) 0x20088C14))  //  Cursor Clip Position register
#define LCD_CRSR_INTMSK  (*((volatile dword *) 0x20088C20))  //  Cursor Interrupt Mask register
#define LCD_CRSR_INTCLR  (*((volatile dword *) 0x20088C24))  //  Cursor Interrupt Clear register WO
#define LCD_CRSR_INTRAW  (*((volatile dword *) 0x20088C28))  //  Cursor Raw Interrupt Status register RO
#define LCD_CRSR_INTSTAT (*((volatile dword *) 0x20088C2C))  //  Cursor Masked Interrupt Status register RO


/************ LPC4088 USB device controller **********************************/
/* base address 0x2008C000 */

/* Port select register */
#define USB_PORTSEL    (*((volatile dword *) 0x2008C110))  //  USB Port Select, also used for OTG

/* Device interrupt registers */
#define USB_DEVINTST   (*((volatile dword *) 0x2008C200))  //  USB Device Interrupt Status RO
#define USB_DEVINTEN   (*((volatile dword *) 0x2008C204))  //  USB Device Interrupt Enable
#define USB_DEVINTCLR  (*((volatile dword *) 0x2008C208))  //  USB Device Interrupt Clear WO
#define USB_DEVINTSET  (*((volatile dword *) 0x2008C20C))  //  USB Device Interrupt Set  WO
#define USB_DEVINTPRI  (*((volatile dword *) 0x2008C22C))  //  USB Device Interrupt Priority WO

/* Endpoint interrupt registers */
#define USB_EPINTST    (*((volatile dword *) 0x2008C230))  //  USB Endpoint Interrupt Status RO
#define USB_EPINTEN    (*((volatile dword *) 0x2008C234))  //  USB Endpoint Interrupt Enable
#define USB_EPINTCLR   (*((volatile dword *) 0x2008C238))  //  USB Endpoint Interrupt Clear WO
#define USB_EPINTSET   (*((volatile dword *) 0x2008C23C))  //  USB Endpoint Interrupt Set WO
#define USB_EPINTPRI   (*((volatile dword *) 0x2008C240))  //  USB Endpoint Priority WO

/* Endpoint realization registers */
#define USB_REEP       (*((volatile dword *) 0x2008C244))  //  USB Realize Endpoint 
#define USB_EPIN       (*((volatile dword *) 0x2008C248))  //  USB Endpoint Index WO
#define USB_MAXPSIZE   (*((volatile dword *) 0x2008C24C))  //  USB MaxPacketSize

/* USB transfer registers */
#define USB_RXDATA     (*((volatile dword *) 0x2008C218))  //  USB Receive Data RO
#define USB_RXPLEN     (*((volatile dword *) 0x2008C220))  //  USB Receive Packet Length RO
#define USB_TXDATA     (*((volatile dword *) 0x2008C21C))  //  USB Transmit Data  WO
#define USB_TXPLEN     (*((volatile dword *) 0x2008C224))  //  USB Transmit Packet Length WO
#define USB_CTRL       (*((volatile dword *) 0x2008C228))  //  USB Control

/* SIE Command registers */
#define USB_CMDCODE    (*((volatile dword *) 0x2008C210))  //  USB Command Code WO
#define USB_CMDDATA    (*((volatile dword *) 0x2008C214))  //  USB Command Data RO

/* DMA registers */
#define USB_DMARST     (*((volatile dword *) 0x2008C250))  //  USB DMA Request Status RO
#define USB_DMARCLR    (*((volatile dword *) 0x2008C254))  //  USB DMA Request Clear WO
#define USB_DMARSET    (*((volatile dword *) 0x2008C258))  //  USB DMA Request Set WO
#define USB_UDCAH      (*((volatile dword *) 0x2008C280))  //  USB UDCA Head
#define USB_EPDMAST    (*((volatile dword *) 0x2008C284))  //  USB Endpoint DMA Status RO
#define USB_EPDMAEN    (*((volatile dword *) 0x2008C288))  //  USB Endpoint DMA Enable WO
#define USB_EPDMADIS   (*((volatile dword *) 0x2008C28C))  //  USB Endpoint DMA Disable WO
#define USB_DMAINTST   (*((volatile dword *) 0x2008C290))  //  USB DMA Interrupt Status RO
#define USB_DMAINTEN   (*((volatile dword *) 0x2008C294))  //  USB DMA Interrupt Enable
#define USB_EOTINTST   (*((volatile dword *) 0x2008C2A0))  //  USB End of Transfer Interrupt Status RO
#define USB_EOTINTCLR  (*((volatile dword *) 0x2008C2A4))  //  USB End of Transfer Interrupt Clear WO
#define USB_EOTINTSET  (*((volatile dword *) 0x2008C2A8))  //  USB End of Transfer Interrupt Set WO

#define USB_NDDRINTST     (*((volatile dword *) 0x2008C2AC))  //  USB New DD Request Interrupt Status RO
#define USB_NDDRINTCLR    (*((volatile dword *) 0x2008C2B0))  //  USB New DD Request Interrupt Clear WO
#define USB_NDDRINTSET    (*((volatile dword *) 0x2008C2B4))  //  USB New DD Request Interrupt Set WO
#define USB_SYSERRINTST   (*((volatile dword *) 0x2008C2B8))  //  USB System Error Interrupt Status RO
#define USB_SYSERRINTCLR  (*((volatile dword *) 0x2008C2BC))  //  USB System Error Interrupt Clear WO
#define USB_SYSERRINTSET  (*((volatile dword *) 0x2008C2C0))  //  USB System Error Interrupt Set WO

/* Clock control registers */
#define USB_CLKCTRL       (*((volatile dword *) 0x2008CFF4))  //  USB Clock Control
#define USB_CLKST         (*((volatile dword *) 0x2008CFF8))  //  USB Clock Status RO


/************ LPC4088 USB OTG controller *************************************/
/* base address 0x2008C000 */

/* OTG registers */
#define USB_OTG_INTST     (*((volatile dword *) 0x2008C100)) //  OTG Interrupt Status
#define USB_OTG_INTEN     (*((volatile dword *) 0x2008C104)) //  OTG Interrupt Enable
#define USB_OTG_INTSET    (*((volatile dword *) 0x2008C108)) //  OTG Interrupt Set
#define USB_OTG_INTCLR    (*((volatile dword *) 0x2008C10C)) //  OTG Interrupt Clear
#define USB_OTG_PORTSEL   (*((volatile dword *) 0x2008C110)) //  OTG Status, siehe USB_PORTSEL
#define USB_OTG_TMR       (*((volatile dword *) 0x2008C114)) //  OTG Timer

/* I2C registers */
#define USB_OTG_I2C_RX    (*((volatile dword *) 0x2008C300)) //  I2C Receive NA 320
#define USB_OTG_I2C_TX    (*((volatile dword *) 0x2008C300)) //  I2C Transmit  NA 321
#define USB_OTG_I2C_STS   (*((volatile dword *) 0x2008C304)) //  I2C Status 0x0A00 322
#define USB_OTG_I2C_CTL   (*((volatile dword *) 0x2008C308)) //  I2C Control 0 323
#define USB_OTG_I2C_CLKHI (*((volatile dword *) 0x2008C30C)) //  I2C Clock High 0xB9 324
#define USB_OTG_I2C_CLKLO (*((volatile dword *) 0x2008C310)) //  I2C Clock Low 0xB9 325

/* Clock control registers */
#define USB_OTG_CLKCTRL   (*((volatile dword *) 0x2008FF4)) //  OTG clock controller 0 326
#define USB_OTG_CLKST     (*((volatile dword *) 0x2008FF8)) //  OTG clock status


/************ LPC4088 USB host controller ************************************/
#define USB_HcRevision         (*((volatile dword *) 0x2008C000)) // BCD of the HCI specification implemented 
#define USB_HcControl          (*((volatile dword *) 0x2008C004)) // Defines the operating modes of the HC
#define USB_HcCommandStatus    (*((volatile dword *) 0x2008C008)) // receive the commands from the Host Controller Driver (HCD)
#define USB_HcInterruptStatus  (*((volatile dword *) 0x2008C00C)) // status on various events that cause hardware interrupts
#define USB_HcInterruptEnable  (*((volatile dword *) 0x2008C010)) // HcInterrupt enables
#define USB_HcInterruptDisable (*((volatile dword *) 0x2008C014)) // HcInterrupt disables
#define USB_HcHCCA             (*((volatile dword *) 0x2008C018)) // address of the host controller communication area
#define USB_HcPeriodCurrentED  (*((volatile dword *) 0x2008C01C)) // address of the current isochronous or interrupt endpoint descriptor
#define USB_HcControlHeadED    (*((volatile dword *) 0x2008C020)) // address of the first endpoint descriptor of the control list
#define USB_HcControlCurrentED (*((volatile dword *) 0x2008C024)) // address of the current endpoint descriptor of the control list 
#define USB_HcBulkHeadED       (*((volatile dword *) 0x2008C028)) // address of the first endpoint descriptor of the bulk list
#define USB_HcBulkCurrentED    (*((volatile dword *) 0x2008C02C)) // address of the current endpoint descriptor of the bulk list
#define USB_HcDoneHead         (*((volatile dword *) 0x2008C030)) // address of the last transfer descriptor added to the Done queue
#define USB_HcFmInterval       (*((volatile dword *) 0x2008C034)) // bit time interval in a frame and full speed max packet size
#define USB_HcFmRemaining      (*((volatile dword *) 0x2008C038)) // 14-bit counter showing the bit time remaining in the current frame. 
#define USB_HcFmNumber         (*((volatile dword *) 0x2008C03C)) // 16-bit counter, timing reference among events happening
#define USB_HcPeriodicStart    (*((volatile dword *) 0x2008C040)) // 14-bit value for start processing a periodic list
#define USB_HcLSThreshold      (*((volatile dword *) 0x2008C044)) // 11-bit value for commit to transfer
#define USB_HcRhDescriptorA    (*((volatile dword *) 0x2008C048)) // First root hub descriptor
#define USB_HcRhDescriptorB    (*((volatile dword *) 0x2008C04C)) // Second root hub descriptor
#define USB_HcRhStatus         (*((volatile dword *) 0x2008C050)) // hub status field and hub status change field
#define USB_HcRhPortStatus_1   (*((volatile dword *) 0x2008C054)) // port events on a per-port basis
#define USB_HcRhPortStatus_2   (*((volatile dword *) 0x2008C058)) // port events on a per port basis
#define USB_Module_ID          (*((volatile dword *) 0x2008C0FC)) // IP number


/************ LPC4088 SPIFI (base address 0x20094000) ************************/

#define SPIFI_CTRL   (*((volatile dword *) 0x20094000)) //  SPIFI control register        0x400F FFFF 
#define SPIFI_CMD    (*((volatile dword *) 0x20094004)) //  SPIFI command register        0x0000 0000 
#define SPIFI_ADDR   (*((volatile dword *) 0x20094008)) //  SPIFI address register        0x0000 0000 
#define SPIFI_IDATA  (*((volatile dword *) 0x2009400C)) //  SPIFI intermediate data register 0x0000 0000 
#define SPIFI_CLIMIT (*((volatile dword *) 0x20094010)) //  SPIFI cache limit register    0x0800 0000 
#define SPIFI_DATA   (*((volatile dword *) 0x20094014)) //  SPIFI data register           0x0000 0000 
#define SPIFI_MCMD   (*((volatile dword *) 0x20094018)) //  SPIFI memory command register 0x0000 0000 
#define SPIFI_STAT   (*((volatile dword *) 0x2009401C)) //  SPIFI status register         0x0200 0000 


/************ LPC4088 SD card interface **************************************/
/* base address 0x400C0000 */
#define MCI_PWR        (*((volatile dword *) 0x400C0000)) //  Power control register
#define MCI_CLOCK      (*((volatile dword *) 0x400C0004)) //  Clock control register
#define MCI_ARGUMENT   (*((volatile dword *) 0x400C0008)) //  Argument register
#define MCI_COMMAND    (*((volatile dword *) 0x400C000C)) //  Command register
#define MCI_RESPCMD    (*((volatile dword *) 0x400C0010)) //  Response command register
#define MCI_RESPONSE0  (*((volatile dword *) 0x400C0014)) //  Response register
#define MCI_RESPONSE1  (*((volatile dword *) 0x400C0018)) //  Response register
#define MCI_RESPONSE2  (*((volatile dword *) 0x400C001C)) //  Response register
#define MCI_RESPONSE3  (*((volatile dword *) 0x400C0020)) //  Response register
#define MCI_DATATIMER  (*((volatile dword *) 0x400C0024)) //  Data Timer
#define MCI_DATALENGTH (*((volatile dword *) 0x400C0028)) //  Data length register
#define MCI_DATACTRL   (*((volatile dword *) 0x400C002C)) //  Data control register
#define MCI_DATACNT    (*((volatile dword *) 0x400C0030)) //  Data counter
#define MCI_STATUS     (*((volatile dword *) 0x400C0034)) //  Status register
#define MCI_CLEAR      (*((volatile dword *) 0x400C0038)) //  Clear register
#define MCI_MASK0      (*((volatile dword *) 0x400C003C)) //  Interrupt 0 mask register
#define MCI_FIFO       (*((volatile dword *) 0x400C0080)) //  0x400C0080 to 0x400C00BC Data FIFO Register.
#define MCI_FIFOCNT    (*((volatile dword *) 0x400C4048)) //  FIFO Counter. 


/************ LPC4088 UART0 **************************************************/
/* base address 0x4000C000 */
#define U0_RBR (*((volatile dword *) 0x4000C000)) // DLAB =0. Receiver Buffer Register
#define U0_THR (*((volatile dword *) 0x4000C000)) // DLAB =0. Transmit Holding Register
#define U0_DLL (*((volatile dword *) 0x4000C000)) // DLAB =1. Divisor Latch LSB
#define U0_DLM (*((volatile dword *) 0x4000C004)) // DLAB =1. Divisor Latch MSB
#define U0_IER (*((volatile dword *) 0x4000C004)) // DLAB =0. Interrupt Enable Register
#define U0_IIR (*((volatile dword *) 0x4000C008)) // Interrupt ID Register
#define U0_FCR (*((volatile dword *) 0x4000C008)) // FIFO Control Register
#define U0_LCR (*((volatile dword *) 0x4000C00C)) // Line Control Register
// kein U0_MCR 
#define U0_LSR (*((volatile dword *) 0x4000C014)) // Line Status Register
// kein U0_MSR
#define U0_SCR (*((volatile dword *) 0x4000C01C)) // Scratch Pad Register
#define U0_ACR (*((volatile dword *) 0x4000C020)) // Auto-baud Control Register
#define U0_FDR (*((volatile dword *) 0x4000C028)) // Fractional Divider Register
#define U0_TER (*((volatile dword *) 0x4000C030)) // Transmit Enable Register

#define U0_RS485CTRL  (*((volatile dword *) 0x4000C04C)) // RS-485/EIA-485 Control
#define U0_RSADRMATCH (*((volatile dword *) 0x4000C050)) // RS-485/EIA-485 address match
#define U0_RS485DLY   (*((volatile dword *) 0x4000C054)) // RS-485/EIA-485 direction control delay.


/************ LPC4088 UART1 **************************************************/
/* base address 0x40010000 */
#define U1_RBR (*((volatile dword *) 0x40010000)) // DLAB =0. Receiver Buffer Register
#define U1_THR (*((volatile dword *) 0x40010000)) // DLAB =0. Transmit Holding Register
#define U1_DLL (*((volatile dword *) 0x40010000)) // DLAB =1. Divisor Latch LSB
#define U1_DLM (*((volatile dword *) 0x40010004)) // DLAB =1. Divisor Latch MSB
#define U1_IER (*((volatile dword *) 0x40010004)) // DLAB =0. Interrupt Enable Register
#define U1_IIR (*((volatile dword *) 0x40010008)) // Interrupt ID Register
#define U1_FCR (*((volatile dword *) 0x40010008)) // FIFO Control Register
#define U1_LCR (*((volatile dword *) 0x4001000C)) // Line Control Register
#define U1_MCR (*((volatile dword *) 0x40010010)) // Modem Control Register
#define U1_LSR (*((volatile dword *) 0x40010014)) // Line Status Register
#define U1_MSR (*((volatile dword *) 0x40010018)) // Modem Status Register
#define U1_SCR (*((volatile dword *) 0x4001001C)) // Scratch Pad Register
#define U1_ACR (*((volatile dword *) 0x40010020)) // Auto-baud Control Register
#define U1_FDR (*((volatile dword *) 0x40010028)) // Fractional Divider Register
#define U1_TER (*((volatile dword *) 0x40010030)) // Transmit Enable Register

#define U1_RS485CTRL  (*((volatile dword *) 0x4001004C)) // RS-485/EIA-485 Control
#define U1_RSADRMATCH (*((volatile dword *) 0x40010050)) // RS-485/EIA-485 address match
#define U1_RS485DLY   (*((volatile dword *) 0x40010054)) // RS-485/EIA-485 direction control delay.


/************ LPC4088 UART2 **************************************************/
/* base address 0x40088000 */
#define U2_RBR (*((volatile dword *) 0x40088000)) // DLAB =0. Receiver Buffer Register
#define U2_THR (*((volatile dword *) 0x40088000)) // DLAB =0. Transmit Holding Register
#define U2_DLL (*((volatile dword *) 0x40088000)) // DLAB =1. Divisor Latch LSB
#define U2_DLM (*((volatile dword *) 0x40088004)) // DLAB =1. Divisor Latch MSB
#define U2_IER (*((volatile dword *) 0x40088004)) // DLAB =0. Interrupt Enable Register
#define U2_IIR (*((volatile dword *) 0x40088008)) // Interrupt ID Register
#define U2_FCR (*((volatile dword *) 0x40088008)) // FIFO Control Register
#define U2_LCR (*((volatile dword *) 0x4008800C)) // Line Control Register
// kein U2_MCR 
#define U2_LSR (*((volatile dword *) 0x40088014)) // Line Status Register
// kein U2_MSR
#define U2_SCR (*((volatile dword *) 0x4008801C)) // Scratch Pad Register
#define U2_ACR (*((volatile dword *) 0x40088020)) // Auto-baud Control Register
#define U2_FDR (*((volatile dword *) 0x40088028)) // Fractional Divider Register
#define U2_TER (*((volatile dword *) 0x40088030)) // Transmit Enable Register

#define U2_RS485CTRL  (*((volatile dword *) 0x4008804C)) // RS-485/EIA-485 Control
#define U2_RSADRMATCH (*((volatile dword *) 0x40088050)) // RS-485/EIA-485 address match
#define U2_RS485DLY   (*((volatile dword *) 0x40088054)) // RS-485/EIA-485 direction control delay.


/************ LPC4088 UART3 **************************************************/
/* base address 0x4009C000 */
#define U3_RBR (*((volatile dword *) 0x4009C000)) // DLAB =0. Receiver Buffer Register
#define U3_THR (*((volatile dword *) 0x4009C000)) // DLAB =0. Transmit Holding Register
#define U3_DLL (*((volatile dword *) 0x4009C000)) // DLAB =1. Divisor Latch LSB
#define U3_DLM (*((volatile dword *) 0x4009C004)) // DLAB =1. Divisor Latch MSB
#define U3_IER (*((volatile dword *) 0x4009C004)) // DLAB =0. Interrupt Enable Register
#define U3_IIR (*((volatile dword *) 0x4009C008)) // Interrupt ID Register
#define U3_FCR (*((volatile dword *) 0x4009C008)) // FIFO Control Register
#define U3_LCR (*((volatile dword *) 0x4009C00C)) // Line Control Register
// kein U3_MCR 
#define U3_LSR (*((volatile dword *) 0x4009C014)) // Line Status Register
// kein U3_MSR
#define U3_SCR (*((volatile dword *) 0x4009C01C)) // Scratch Pad Register
#define U3_ACR (*((volatile dword *) 0x4009C020)) // Auto-baud Control Register
#define U3_FDR (*((volatile dword *) 0x4009C028)) // Fractional Divider Register
#define U3_TER (*((volatile dword *) 0x4009C030)) // Transmit Enable Register

#define U3_RS485CTRL  (*((volatile dword *) 0x4009C04C)) // RS-485/EIA-485 Control
#define U3_RSADRMATCH (*((volatile dword *) 0x4009C050)) // RS-485/EIA-485 address match
#define U3_RS485DLY   (*((volatile dword *) 0x4009C054)) // RS-485/EIA-485 direction control delay.

/************ LPC4088 UART4 **************************************************/
/* base address 0x400A4000 */
#define U4_RBR  (*((volatile dword *) 0x400A4000)) // Receiver Buffer Register
#define U4_THR  (*((volatile dword *) 0x400A4000)) // Transmit Holding Register
#define U4_DLL  (*((volatile dword *) 0x400A4000)) // Divisor Latch LSB
#define U4_DLM  (*((volatile dword *) 0x400A4004)) // Divisor Latch MSB
#define U4_IER  (*((volatile dword *) 0x400A4004)) // Interrupt Enable Register
#define U4_IIR  (*((volatile dword *) 0x400A4008)) // Interrupt ID Register
#define U4_FCR  (*((volatile dword *) 0x400A4008)) // FIFO Control Register
#define U4_LCR  (*((volatile dword *) 0x400A400C)) // Line Control Register
#define U4_LSR  (*((volatile dword *) 0x400A4014)) // Line Status Register
#define U4_SCR  (*((volatile dword *) 0x400A401C)) // Scratch Pad Register
#define U4_ACR  (*((volatile dword *) 0x400A4020)) // Auto-baud Control Register
#define U4_ICR  (*((volatile dword *) 0x400A4024)) // IrDA Control Register
#define U4_FDR  (*((volatile dword *) 0x400A4028)) // Fractional Divider Register
#define U4_OSR  (*((volatile dword *) 0x400A402C)) // Oversampling register

#define U4_SCICTRL   (*((volatile dword *) 0x400A4048)) // Smart card Interface control register
#define U4_RS485CTRL (*((volatile dword *) 0x400A404C)) // RS-485/EIA-485 Control
#define U4_ADRMATCH  (*((volatile dword *) 0x400A4050)) // RS-485/EIA-485 address match
#define U4_RS485DLY  (*((volatile dword *) 0x400A4054)) // RS-485/EIA-485 direction control delay
#define U4_SYNCCTRL  (*((volatile dword *) 0x400A4058)) // Synchronous mode control register


/************ LPC4088 CAN ****************************************************/
/* CAN acceptance filter (base address 0x4003C000) */
#define CAN_AFMR       (*((volatile dword *) 0x4003C000)) // Acceptance Filter Register
#define CAN_SFF_SA     (*((volatile dword *) 0x4003C004)) // Standard Frame Individual Start Address Register
#define CAN_SFF_GRP_SA (*((volatile dword *) 0x4003C008)) // Standard Frame Group Start Address Register
#define CAN_EFF_SA     (*((volatile dword *) 0x4003C00C)) // Extended Frame Start Address Register
#define CAN_EFF_GRP_SA (*((volatile dword *) 0x4003C010)) // Extended Frame Group Start Address Register
#define CAN_ENDOFTABLE (*((volatile dword *) 0x4003C014)) // End of AF Tables register
#define CAN_LUTERRAD   (*((volatile dword *) 0x4003C018)) // LUT Error Address register
#define CAN_LUTERR     (*((volatile dword *) 0x4003C01C)) // LUT Error Register
#define CAN_FCANIE     (*((volatile dword *) 0x4003C020)) // FullCAN interrupt enable register
#define CAN_FCANIC0    (*((volatile dword *) 0x4003C024)) // FullCAN interrupt and capture register0 
#define CAN_FCANIC1    (*((volatile dword *) 0x4003C028)) // FullCAN interrupt and capture register1

/* central CAN (base address 0x40040000) */
#define CAN_TXSR (*((volatile dword *) 0x40044000)) // CAN Central Transmit Status Register
#define CAN_RXSR (*((volatile dword *) 0x40044004)) // CAN Central Receive Status Register 
#define CAN_MSR  (*((volatile dword *) 0x40044008)) // CAN Central Miscellaneous Register



/* CAN1 (base address 0x40044000) */
#define CAN1_MOD  (*((volatile dword *) 0x40044000)) // Controls the operating mode of the CAN Controller
#define CAN1_CMR  (*((volatile dword *) 0x40044004)) // Command bits that affect the state of the CAN Controller 
#define CAN1_GSR  (*((volatile dword *) 0x40044008)) // Global Controller Status and Error Counters.
#define CAN1_ICR  (*((volatile dword *) 0x4004400C)) // Interrupt status, Arbitration Lost Capture, Error Code Capture 
#define CAN1_IER  (*((volatile dword *) 0x40044010)) // Interrupt Enable 0 446
#define CAN1_BTR  (*((volatile dword *) 0x40044014)) // Bus Timing. Can only be written when RM in CANMOD is 1
#define CAN1_EWL  (*((volatile dword *) 0x40044018)) // Error Warning Limit. Can only be written when RM in CANMOD is 1
#define CAN1_SR   (*((volatile dword *) 0x4004401C)) // Status Register 0x3C3C3C 449
#define CAN1_RFS  (*((volatile dword *) 0x40044020)) // Receive frame status. Can only be written when RM in CANMOD is 1
#define CAN1_RID  (*((volatile dword *) 0x40044024)) // Received Identifier. Can only be written when RM in CANMOD is 1
#define CAN1_RDA  (*((volatile dword *) 0x40044028)) // Received data bytes 1-4. Can only be written when RM in CANMOD is 1.
#define CAN1_RDB  (*((volatile dword *) 0x4004402C)) // Received data bytes 5-8. Can only be written when RM in CANMOD is 1.
#define CAN1_TFI1 (*((volatile dword *) 0x40044030)) // Transmit frame info (Tx Buffer 1)
#define CAN1_TID1 (*((volatile dword *) 0x40044034)) // Transmit Identifier (Tx Buffer 1)
#define CAN1_TDA1 (*((volatile dword *) 0x40044038)) // Transmit data bytes 1-4 (Tx Buffer 1) 
#define CAN1_TDB1 (*((volatile dword *) 0x4004403C)) // Transmit data bytes 5-8 (Tx Buffer 1) 
#define CAN1_TFI2 (*((volatile dword *) 0x40044040)) // Transmit frame info (Tx Buffer 2) 
#define CAN1_TID2 (*((volatile dword *) 0x40044044)) // Transmit Identifier (Tx Buffer 2) 
#define CAN1_TDA2 (*((volatile dword *) 0x40044048)) // Transmit data bytes 1-4 (Tx Buffer 2) 
#define CAN1_TDB2 (*((volatile dword *) 0x4004404C)) // Transmit data bytes 5-8 (Tx Buffer 2) 
#define CAN1_TFI3 (*((volatile dword *) 0x40044050)) // Transmit frame info (Tx Buffer 3) 
#define CAN1_TID3 (*((volatile dword *) 0x40044054)) // Transmit Identifier (Tx Buffer 3) 
#define CAN1_TDA3 (*((volatile dword *) 0x40044058)) // Transmit data bytes 1-4 (Tx Buffer 3)
#define CAN1_TDB3 (*((volatile dword *) 0x4004405C)) // Transmit data bytes 5-8 (Tx Buffer 3)


/* CAN2 (base address 0x40048000) */
#define CAN2_MOD  (*((volatile dword *) 0x40048000)) // Controls the operating mode of the CAN Controller
#define CAN2_CMR  (*((volatile dword *) 0x40048004)) // Command bits that affect the state of the CAN Controller 
#define CAN2_GSR  (*((volatile dword *) 0x40048008)) // Global Controller Status and Error Counters.
#define CAN2_ICR  (*((volatile dword *) 0x4004800C)) // Interrupt status, Arbitration Lost Capture, Error Code Capture 
#define CAN2_IER  (*((volatile dword *) 0x40048010)) // Interrupt Enable 0 446
#define CAN2_BTR  (*((volatile dword *) 0x40048014)) // Bus Timing. Can only be written when RM in CANMOD is 1
#define CAN2_EWL  (*((volatile dword *) 0x40048018)) // Error Warning Limit. Can only be written when RM in CANMOD is 1
#define CAN2_SR   (*((volatile dword *) 0x4004801C)) // Status Register 0x3C3C3C 449
#define CAN2_RFS  (*((volatile dword *) 0x40048020)) // Receive frame status. Can only be written when RM in CANMOD is 1
#define CAN2_RID  (*((volatile dword *) 0x40048024)) // Received Identifier. Can only be written when RM in CANMOD is 1
#define CAN2_RDA  (*((volatile dword *) 0x40048028)) // Received data bytes 1-4. Can only be written when RM in CANMOD is 1.
#define CAN2_RDB  (*((volatile dword *) 0x4004802C)) // Received data bytes 5-8. Can only be written when RM in CANMOD is 1.
#define CAN2_TFI1 (*((volatile dword *) 0x40048030)) // Transmit frame info (Tx Buffer 1)
#define CAN2_TID1 (*((volatile dword *) 0x40048034)) // Transmit Identifier (Tx Buffer 1)
#define CAN2_TDA1 (*((volatile dword *) 0x40048038)) // Transmit data bytes 1-4 (Tx Buffer 1) 
#define CAN2_TDB1 (*((volatile dword *) 0x4004803C)) // Transmit data bytes 5-8 (Tx Buffer 1) 
#define CAN2_TFI2 (*((volatile dword *) 0x40048040)) // Transmit frame info (Tx Buffer 2) 
#define CAN2_TID2 (*((volatile dword *) 0x40048044)) // Transmit Identifier (Tx Buffer 2) 
#define CAN2_TDA2 (*((volatile dword *) 0x40048048)) // Transmit data bytes 1-4 (Tx Buffer 2) 
#define CAN2_TDB2 (*((volatile dword *) 0x4004804C)) // Transmit data bytes 5-8 (Tx Buffer 2) 
#define CAN2_TFI3 (*((volatile dword *) 0x40048050)) // Transmit frame info (Tx Buffer 3) 
#define CAN2_TID3 (*((volatile dword *) 0x40048054)) // Transmit Identifier (Tx Buffer 3) 
#define CAN2_TDA3 (*((volatile dword *) 0x40048058)) // Transmit data bytes 1-4 (Tx Buffer 3)
#define CAN2_TDB3 (*((volatile dword *) 0x4004805C)) // Transmit data bytes 5-8 (Tx Buffer 3)


/************ LPC4088 SSP0 ***************************************************/
/* base address 0x40088000 */
#define SSP0_CR0   (*((volatile dword *) 0x40088000)) // Control Register 0
#define SSP0_CR1   (*((volatile dword *) 0x40088004)) // Control Register 1
#define SSP0_DR    (*((volatile dword *) 0x40088008)) // Data Register. transmit FIFO, receive FIFO
#define SSP0_SR    (*((volatile dword *) 0x4008800C)) // Status Register 
#define SSP0_CPSR  (*((volatile dword *) 0x40088010)) // Clock Prescale Register
#define SSP0_IMSC  (*((volatile dword *) 0x40088014)) // Interrupt Mask Set and Clear Register 
#define SSP0_RIS   (*((volatile dword *) 0x40088018)) // Raw Interrupt Status Register 
#define SSP0_MIS   (*((volatile dword *) 0x4008801C)) // Masked Interrupt Status Register 
#define SSP0_ICR   (*((volatile dword *) 0x40088020)) // SSPICR Interrupt Clear Register 
#define SSP0_DMACR (*((volatile dword *) 0x40088024)) // DMA Control Register


/************ LPC4088 SSP1 ***************************************************/
/* base address 0x40030000 */
#define SSP1_CR0   (*((volatile dword *) 0x40030000)) // Control Register 0
#define SSP1_CR1   (*((volatile dword *) 0x40030004)) // Control Register 1
#define SSP1_DR    (*((volatile dword *) 0x40030008)) // Data Register. transmit FIFO, receive FIFO
#define SSP1_SR    (*((volatile dword *) 0x4003000C)) // Status Register 
#define SSP1_CPSR  (*((volatile dword *) 0x40030010)) // Clock Prescale Register
#define SSP1_IMSC  (*((volatile dword *) 0x40030014)) // Interrupt Mask Set and Clear Register 
#define SSP1_RIS   (*((volatile dword *) 0x40030018)) // Raw Interrupt Status Register 
#define SSP1_MIS   (*((volatile dword *) 0x4003001C)) // Masked Interrupt Status Register 
#define SSP1_ICR   (*((volatile dword *) 0x40030020)) // SSPICR Interrupt Clear Register 
#define SSP1_DMACR (*((volatile dword *) 0x40030024)) // DMA Control Register


/************ LPC4088 SSP2 ***************************************************/
/* base address 0x400AC000 */
#define SSP2_CR0   (*((volatile dword *) 0x400AC000)) // Control Register 0
#define SSP2_CR1   (*((volatile dword *) 0x400AC004)) // Control Register 1
#define SSP2_DR    (*((volatile dword *) 0x400AC008)) // Data Register. transmit FIFO, receive FIFO
#define SSP2_SR    (*((volatile dword *) 0x400AC00C)) // Status Register 
#define SSP2_CPSR  (*((volatile dword *) 0x400AC010)) // Clock Prescale Register
#define SSP2_IMSC  (*((volatile dword *) 0x400AC014)) // Interrupt Mask Set and Clear Register 
#define SSP2_RIS   (*((volatile dword *) 0x400AC018)) // Raw Interrupt Status Register 
#define SSP2_MIS   (*((volatile dword *) 0x400AC01C)) // Masked Interrupt Status Register 
#define SSP2_ICR   (*((volatile dword *) 0x400AC020)) // SSPICR Interrupt Clear Register 
#define SSP2_DMACR (*((volatile dword *) 0x400AC024)) // DMA Control Register


/************ LPC4088 I2C0 ***************************************************/
/* base address 0x4001C000 */ 
#define I2C0_CONSET (*((volatile dword *) 0x4001C000)) //  I2C Control Set Register. 
#define I2C0_STAT   (*((volatile dword *) 0x4001C004)) //  I2C Status Register. 9
#define I2C0_DAT    (*((volatile dword *) 0x4001C008)) //  I2C Data Register. 
#define I2C0_ADR0   (*((volatile dword *) 0x4001C00C)) //  I2C Slave Address Register 0. 
#define I2C0_SCLH   (*((volatile dword *) 0x4001C010)) //  SCH Duty Cycle Register High Half Word. 
#define I2C0_SCLL   (*((volatile dword *) 0x4001C014)) //  SCL Duty Cycle Register Low Half Word. 
#define I2C0_CONCLR (*((volatile dword *) 0x4001C018)) //  I2C Control Clear Register. 
#define I2C0_MMCTRL (*((volatile dword *) 0x4001C01C)) //  Monitor mode control register.
#define I2C0_ADR1   (*((volatile dword *) 0x4001C020)) //  I2C Slave Address Register 1. 
#define I2C0_ADR2   (*((volatile dword *) 0x4001C024)) //  I2C Slave Address Register 2. 
#define I2C0_ADR3   (*((volatile dword *) 0x4001C028)) //  I2C Slave Address Register 3. 
#define I2C0_DATA_BUFFER (*((volatile dword *) 0x4001C02C)) //  Data buffer register. 
#define I2C0_MASK0  (*((volatile dword *) 0x4001C030)) //  I2C Slave address mask register 0. 
#define I2C0_MASK1  (*((volatile dword *) 0x4001C034)) //  I2C Slave address mask register 1. 
#define I2C0_MASK2  (*((volatile dword *) 0x4001C038)) //  I2C Slave address mask register 2. 
#define I2C0_MASK3  (*((volatile dword *) 0x4001C03C)) //  I2C Slave address mask register 3. 

/************ LPC4088 I2C1 ***************************************************/
/* base address 0x4005C000 */
#define I2C1_CONSET (*((volatile dword *) 0x4005C000)) //  I2C Control Set Register. 
#define I2C1_STAT   (*((volatile dword *) 0x4005C004)) //  I2C Status Register. 9
#define I2C1_DAT    (*((volatile dword *) 0x4005C008)) //  I2C Data Register. 
#define I2C1_ADR0   (*((volatile dword *) 0x4005C00C)) //  I2C Slave Address Register 0. 
#define I2C1_SCLH   (*((volatile dword *) 0x4005C010)) //  SCH Duty Cycle Register High Half Word. 
#define I2C1_SCLL   (*((volatile dword *) 0x4005C014)) //  SCL Duty Cycle Register Low Half Word. 
#define I2C1_CONCLR (*((volatile dword *) 0x4005C018)) //  I2C Control Clear Register. 
#define I2C1_MMCTRL (*((volatile dword *) 0x4005C01C)) //  Monitor mode control register.
#define I2C1_ADR1   (*((volatile dword *) 0x4005C020)) //  I2C Slave Address Register 1. 
#define I2C1_ADR2   (*((volatile dword *) 0x4005C024)) //  I2C Slave Address Register 2. 
#define I2C1_ADR3   (*((volatile dword *) 0x4005C028)) //  I2C Slave Address Register 3. 
#define I2C1_DATA_BUFFER (*((volatile dword *) 0x4005C02C)) //  Data buffer register. 
#define I2C1_MASK0  (*((volatile dword *) 0x4005C030)) //  I2C Slave address mask register 0. 
#define I2C1_MASK1  (*((volatile dword *) 0x4005C034)) //  I2C Slave address mask register 1. 
#define I2C1_MASK2  (*((volatile dword *) 0x4005C038)) //  I2C Slave address mask register 2. 
#define I2C1_MASK3  (*((volatile dword *) 0x4005C03C)) //  I2C Slave address mask register 3. 


/************ LPC4088 I2C2 ***************************************************/
/* base address 0x400A0000 */
#define I2C2_CONSET (*((volatile dword *) 0x400A0000)) //  I2C Control Set Register. 
#define I2C2_STAT   (*((volatile dword *) 0x400A0004)) //  I2C Status Register. 9
#define I2C2_DAT    (*((volatile dword *) 0x400A0008)) //  I2C Data Register. 
#define I2C2_ADR0   (*((volatile dword *) 0x400A000C)) //  I2C Slave Address Register 0. 
#define I2C2_SCLH   (*((volatile dword *) 0x400A0010)) //  SCH Duty Cycle Register High Half Word. 
#define I2C2_SCLL   (*((volatile dword *) 0x400A0014)) //  SCL Duty Cycle Register Low Half Word. 
#define I2C2_CONCLR (*((volatile dword *) 0x400A0018)) //  I2C Control Clear Register. 
#define I2C2_MMCTRL (*((volatile dword *) 0x400A001C)) //  Monitor mode control register.
#define I2C2_ADR1   (*((volatile dword *) 0x400A0020)) //  I2C Slave Address Register 1. 
#define I2C2_ADR2   (*((volatile dword *) 0x400A0024)) //  I2C Slave Address Register 2. 
#define I2C2_ADR3   (*((volatile dword *) 0x400A0028)) //  I2C Slave Address Register 3. 
#define I2C2_DATA_BUFFER (*((volatile dword *) 0x400A002C)) //  Data buffer register. 
#define I2C2_MASK0  (*((volatile dword *) 0x400A0030)) //  I2C Slave address mask register 0. 
#define I2C2_MASK1  (*((volatile dword *) 0x400A0034)) //  I2C Slave address mask register 1. 
#define I2C2_MASK2  (*((volatile dword *) 0x400A0038)) //  I2C Slave address mask register 2. 
#define I2C2_MASK3  (*((volatile dword *) 0x400A003C)) //  I2C Slave address mask register 3. 


/************ LPC4088 I2S ****************************************************/
/* base address 0x400A8000 */
#define I2S_DAO       (*((volatile dword *) 0x400A8000)) //  Digital Audio Output Register. Contains control bits for the I2S transmit channel. 0x87E1 519
#define I2S_DAI       (*((volatile dword *) 0x400A8004)) //  Digital Audio Input Register. Contains control bits for the I2S receive channel. 0x07E1 520
#define I2S_TXFIFO    (*((volatile dword *) 0x400A8008)) //  Transmit FIFO. Access register for the 8 ? 32-bit transmitter FIFO. 0 521
#define I2S_RXFIFO    (*((volatile dword *) 0x400A800C)) //  Receive FIFO. Access register for the 8 ? 32-bit receiver FIFO. 0 522
#define I2S_STATE     (*((volatile dword *) 0x400A8010)) //  Status Feedback Register. Contains status information about the I2S interface. 0x7 523
#define I2S_DMA1      (*((volatile dword *) 0x400A8014)) //  DMA Configuration Register 1. Contains control information for DMA request 1. 0 524
#define I2S_DMA2      (*((volatile dword *) 0x400A8018)) //  DMA Configuration Register 2. Contains control information for DMA request 2. 0 525
#define I2S_IRQ       (*((volatile dword *) 0x400A801C)) //  Interrupt Request Control Register. Contains bits that control how  the I2S interrupt request is generated. 0 526
#define I2S_TXRATE    (*((volatile dword *) 0x400A8020)) //  Transmit reference clock divider. This register determines the I2S TX_REF rate by specifying the value to divide CCLK by in order to  produce TX_REF. 0 527
#define I2S_RXRATE    (*((volatile dword *) 0x400A8024)) //  Receive reference clock divider. This register determines the I2S RX_REF rate by specifying the value to divide CCLK by in order to produce RX_REF. 0 528
#define I2S_TXBITRATE (*((volatile dword *) 0x400A8028)) //  Transmit bit rate divider. This register determines the I2S transmit bit rate by specifying the value to divide TX_REF by in order to  produce the transmit bit clock.
#define I2S_RXBITRATE (*((volatile dword *) 0x400A802C)) //  Receive bit rate divider. This register determines the I2S receive bit rate by specifying the value to divide RX_REF by in order to produce the receive bit clock.
#define I2S_TXMODE    (*((volatile dword *) 0x400A8030)) //  Transmit mode control. 
#define I2S_RXMODE    (*((volatile dword *) 0x400A8034)) //  Receive mode control.


/************ LPC4088 Timer0 *************************************************/
/* base address 0x40004000 */
#define T0_IR    (*((volatile dword *) 0x40004000)) //  Interrupt Register
#define T0_TCR   (*((volatile dword *) 0x40004004)) //  Timer Control Register
#define T0_TC    (*((volatile dword *) 0x40004008)) //  Timer Counter
#define T0_PR    (*((volatile dword *) 0x4000400C)) //  Prescale Register
#define T0_PC    (*((volatile dword *) 0x40004010)) //  Prescale Counter
#define T0_MCR   (*((volatile dword *) 0x40004014)) //  Match Control Register
#define T0_MR0   (*((volatile dword *) 0x40004018)) //  Match Register 0
#define T0_MR1   (*((volatile dword *) 0x4000401C)) //  Match Register 1
#define T0_MR2   (*((volatile dword *) 0x40004020)) //  Match Register 2
#define T0_MR3   (*((volatile dword *) 0x40004024)) //  Match Register 3
#define T0_CCR   (*((volatile dword *) 0x40004028)) //  Capture Control Register
#define T0_CR0   (*((volatile dword *) 0x4000402C)) //  Capture Register 0  RO
#define T0_CR1   (*((volatile dword *) 0x40004030)) //  Capture Register 1  RO
#define T0_EMR   (*((volatile dword *) 0x4000403C)) //  External Match Register
#define T0_CTCR  (*((volatile dword *) 0x40004070)) //  Count Control Register


/************ LPC4088 Timer1 *************************************************/
/* base address 0x40008000 */
#define T1_IR    (*((volatile dword *) 0x40008000)) //  Interrupt Register
#define T1_TCR   (*((volatile dword *) 0x40008004)) //  Timer Control Register
#define T1_TC    (*((volatile dword *) 0x40008008)) //  Timer Counter
#define T1_PR    (*((volatile dword *) 0x4000800C)) //  Prescale Register
#define T1_PC    (*((volatile dword *) 0x40008010)) //  Prescale Counter
#define T1_MCR   (*((volatile dword *) 0x40008014)) //  Match Control Register
#define T1_MR0   (*((volatile dword *) 0x40008018)) //  Match Register 0
#define T1_MR1   (*((volatile dword *) 0x4000801C)) //  Match Register 1
#define T1_MR2   (*((volatile dword *) 0x40008020)) //  Match Register 2
#define T1_MR3   (*((volatile dword *) 0x40008024)) //  Match Register 3
#define T1_CCR   (*((volatile dword *) 0x40008028)) //  Capture Control Register
#define T1_CR0   (*((volatile dword *) 0x4000802C)) //  Capture Register 0  RO
#define T1_CR1   (*((volatile dword *) 0x40008030)) //  Capture Register 1  RO
#define T1_EMR   (*((volatile dword *) 0x4000803C)) //  External Match Register
#define T1_CTCR  (*((volatile dword *) 0x40008070)) //  Count Control Register


/************ LPC4088 Timer2 *************************************************/
/* base address 0x40090000 */
#define T2_IR    (*((volatile dword *) 0x40090000)) //  Interrupt Register
#define T2_TCR   (*((volatile dword *) 0x40090004)) //  Timer Control Register
#define T2_TC    (*((volatile dword *) 0x40090008)) //  Timer Counter
#define T2_PR    (*((volatile dword *) 0x4009000C)) //  Prescale Register
#define T2_PC    (*((volatile dword *) 0x40090010)) //  Prescale Counter
#define T2_MCR   (*((volatile dword *) 0x40090014)) //  Match Control Register
#define T2_MR0   (*((volatile dword *) 0x40090018)) //  Match Register 0
#define T2_MR1   (*((volatile dword *) 0x4009001C)) //  Match Register 1
#define T2_MR2   (*((volatile dword *) 0x40090020)) //  Match Register 2
#define T2_MR3   (*((volatile dword *) 0x40090024)) //  Match Register 3
#define T2_CCR   (*((volatile dword *) 0x40090028)) //  Capture Control Register
#define T2_CR0   (*((volatile dword *) 0x4009002C)) //  Capture Register 0  RO
#define T2_CR1   (*((volatile dword *) 0x40090030)) //  Capture Register 1  RO
#define T2_EMR   (*((volatile dword *) 0x4009003C)) //  External Match Register
#define T2_CTCR  (*((volatile dword *) 0x40090070)) //  Count Control Register


/************ LPC4088 Timer3 *************************************************/
/* base address 0x40094000 */
#define T3_IR    (*((volatile dword *) 0x40094000)) //  Interrupt Register
#define T3_TCR   (*((volatile dword *) 0x40094004)) //  Timer Control Register
#define T3_TC    (*((volatile dword *) 0x40094008)) //  Timer Counter
#define T3_PR    (*((volatile dword *) 0x4009400C)) //  Prescale Register
#define T3_PC    (*((volatile dword *) 0x40094010)) //  Prescale Counter
#define T3_MCR   (*((volatile dword *) 0x40094014)) //  Match Control Register
#define T3_MR0   (*((volatile dword *) 0x40094018)) //  Match Register 0
#define T3_MR1   (*((volatile dword *) 0x4009401C)) //  Match Register 1
#define T3_MR2   (*((volatile dword *) 0x40094020)) //  Match Register 2
#define T3_MR3   (*((volatile dword *) 0x40094024)) //  Match Register 3
#define T3_CCR   (*((volatile dword *) 0x40094028)) //  Capture Control Register
#define T3_CR0   (*((volatile dword *) 0x4009402C)) //  Capture Register 0  RO
#define T3_CR1   (*((volatile dword *) 0x40094030)) //  Capture Register 1  RO
#define T3_EMR   (*((volatile dword *) 0x4009403C)) //  External Match Register
#define T3_CTCR  (*((volatile dword *) 0x40094070)) //  Count Control Register


/************ LPC4088  PWM0 **************************************************/
/* base address 0x40014000 */
#define PWM0_IR    (*((volatile dword *) 0x40014000)) //  Interrupt Register
#define PWM0_TCR   (*((volatile dword *) 0x40014004)) //  Timer Control Register
#define PWM0_TC    (*((volatile dword *) 0x40014008)) //  Timer Counter
#define PWM0_PR    (*((volatile dword *) 0x4001400C)) //  Prescale Register
#define PWM0_PC    (*((volatile dword *) 0x40014010)) //  Prescale Counter
#define PWM0_MCR   (*((volatile dword *) 0x40014014)) //  Match Control Register
#define PWM0_MR0   (*((volatile dword *) 0x40014018)) //  Match Register 0
#define PWM0_MR1   (*((volatile dword *) 0x4001401C)) //  Match Register 1. See MR0 description. 0 26.6.7
#define PWM0_MR2   (*((volatile dword *) 0x40014020)) //  Match Register 2. See MR0 description. 0 26.6.7
#define PWM0_MR3   (*((volatile dword *) 0x40014024)) //  Match Register 3. See MR0 description. 0 26.6.7
#define PWM0_CCR   (*((volatile dword *) 0x40014028)) //  Capture Control Register
#define PWM0_CR0   (*((volatile dword *) 0x4001402C)) //  Capture Register 0  RO
#define PWM0_CR1   (*((volatile dword *) 0x40014030)) //  Capture Register 1  RO
#define PWM0_MR4   (*((volatile dword *) 0x40014040)) //  Match Register 4
#define PWM0_MR5   (*((volatile dword *) 0x40014044)) //  Match Register 5
#define PWM0_MR6   (*((volatile dword *) 0x40014048)) //  Match Register 6
#define PWM0_PCR   (*((volatile dword *) 0x4001404C)) //  PWM Control Register
#define PWM0_LER   (*((volatile dword *) 0x40014050)) //  Load Enable Register
#define PWM0_CTCR  (*((volatile dword *) 0x40014070)) //  Count Control Register


/************ LPC4088  PWM1 **************************************************/
/* base address 0x40018000 */
#define PWM1_IR    (*((volatile dword *) 0x40018000)) //  Interrupt Register
#define PWM1_TCR   (*((volatile dword *) 0x40018004)) //  Timer Control Register
#define PWM1_TC    (*((volatile dword *) 0x40018008)) //  Timer Counter
#define PWM1_PR    (*((volatile dword *) 0x4001800C)) //  Prescale Register
#define PWM1_PC    (*((volatile dword *) 0x40018010)) //  Prescale Counter
#define PWM1_MCR   (*((volatile dword *) 0x40018014)) //  Match Control Register
#define PWM1_MR0   (*((volatile dword *) 0x40018018)) //  Match Register 0
#define PWM1_MR1   (*((volatile dword *) 0x4001801C)) //  Match Register 1. See MR0 description. 0 26.6.7
#define PWM1_MR2   (*((volatile dword *) 0x40018020)) //  Match Register 2. See MR0 description. 0 26.6.7
#define PWM1_MR3   (*((volatile dword *) 0x40018024)) //  Match Register 3. See MR0 description. 0 26.6.7
#define PWM1_CCR   (*((volatile dword *) 0x40018028)) //  Capture Control Register
#define PWM1_CR0   (*((volatile dword *) 0x4001802C)) //  Capture Register 0  RO
#define PWM1_CR1   (*((volatile dword *) 0x40018030)) //  Capture Register 1  RO
#define PWM1_MR4   (*((volatile dword *) 0x40018040)) //  Match Register 4
#define PWM1_MR5   (*((volatile dword *) 0x40018044)) //  Match Register 5
#define PWM1_MR6   (*((volatile dword *) 0x40018048)) //  Match Register 6
#define PWM1_PCR   (*((volatile dword *) 0x4001804C)) //  PWM Control Register
#define PWM1_LER   (*((volatile dword *) 0x40018050)) //  Load Enable Register
#define PWM1_CTCR  (*((volatile dword *) 0x40018070)) //  Count Control Register


/************ LPC4088  Motor Control Pulse Width Modulator (MCPWM) ***********/
/* base address 0x400B8000 */
#define MCPWM_CON        (*((volatile dword *) 0x400B8000)) //  PWM Control read address RO
#define MCPWM_CON_SET    (*((volatile dword *) 0x400B8004)) //  PWM Control set address WO
#define MCPWM_CON_CLR    (*((volatile dword *) 0x400B8008)) //  PWM Control clear address WO
#define MCPWM_CAPCON     (*((volatile dword *) 0x400B800C)) //  Capture Control read address RO
#define MCPWM_CAPCON_SET (*((volatile dword *) 0x400B8010)) //  Capture Control set address WO
#define MCPWM_CAPCON_CLR (*((volatile dword *) 0x400B8014)) //  Event Control clear address WO
#define MCPWM_TC0        (*((volatile dword *) 0x400B8018)) //  Timer Counter register, channel 0
#define MCPWM_TC1        (*((volatile dword *) 0x400B801C)) //  Timer Counter register, channel 1
#define MCPWM_TC2        (*((volatile dword *) 0x400B8020)) //  Timer Counter register, channel 2
#define MCPWM_LIM0       (*((volatile dword *) 0x400B8024)) //  Limit register, channel 0
#define MCPWM_LIM1       (*((volatile dword *) 0x400B8028)) //  Limit register, channel 1
#define MCPWM_LIM2       (*((volatile dword *) 0x400B802C)) //  Limit register, channel 2
#define MCPWM_MAT0       (*((volatile dword *) 0x400B8030)) //  Match register, channel 0
#define MCPWM_MAT1       (*((volatile dword *) 0x400B8034)) //  Match register, channel 1
#define MCPWM_MAT2       (*((volatile dword *) 0x400B8038)) //  Match register, channel 2
#define MCPWM_DT         (*((volatile dword *) 0x400B803C)) //  Dead time register
#define MCPWM_MCCP       (*((volatile dword *) 0x400B8040)) //  Communication Pattern register
#define MCPWM_CAP0       (*((volatile dword *) 0x400B8044)) //  Capture register, channel 0 RO
#define MCPWM_CAP1       (*((volatile dword *) 0x400B8048)) //  Capture register, channel 1 RO
#define MCPWM_CAP2       (*((volatile dword *) 0x400B804C)) //  Capture register, channel 2 RO
#define MCPWM_INTEN      (*((volatile dword *) 0x400B8050)) //  Interrupt Enable read address RO
#define MCPWM_INTEN_SET  (*((volatile dword *) 0x400B8054)) //  Interrupt Enable set address WO
#define MCPWM_INTEN_CLR  (*((volatile dword *) 0x400B8058)) //  Interrupt Enable clear address WO
#define MCPWM_CNTCON     (*((volatile dword *) 0x400B805C)) //  Count Control read address RO
#define MCPWM_CNTCON_SET (*((volatile dword *) 0x400B8060)) //  Count Control set address WO
#define MCPWM_CNTCON_CLR (*((volatile dword *) 0x400B8064)) //  Count Control clear address WO
#define MCPWM_INTF       (*((volatile dword *) 0x400B8068)) //  Interrupt flags read address RO
#define MCPWM_INTF_SET   (*((volatile dword *) 0x400B806C)) //  Interrupt flags set address WO
#define MCPWM_INTF_CLR   (*((volatile dword *) 0x400B8070)) //  Interrupt flags clear address WO
#define MCPWM_CAP_CLR    (*((volatile dword *) 0x400B8074)) //  Capture clear address WO


/************ LPC4088  Quadrature Encoder Interface (QEI) ********************/
/* base address 0x400BC000 */
/* Control registers */
#define QEI_CON   (*((volatile dword *) 0x400BC000)) //  Control register WO
#define QEI_CONF  (*((volatile dword *) 0x400BC008)) //  Configuration register
#define QEI_STAT  (*((volatile dword *) 0x400BC004)) //  Status register RO

/* Position, index, and timer registers */
#define QEI_POS       (*((volatile dword *) 0x400BC00C)) //  Position register RO
#define QEI_MAXPOS    (*((volatile dword *) 0x400BC010)) //  Maximum position register
#define QEI_CMPOS0    (*((volatile dword *) 0x400BC014)) //  Position compare register 0
#define QEI_CMPOS1    (*((volatile dword *) 0x400BC018)) //  Position compare register 1
#define QEI_CMPOS2    (*((volatile dword *) 0x400BC01C)) //  Position compare register 2
#define QEI_INXCNT    (*((volatile dword *) 0x400BC020)) //  Index count register RO
#define QEI_INXCMP0   (*((volatile dword *) 0x400BC024)) //  Index compare register 0
#define QEI_INXCMP1   (*((volatile dword *) 0x400BC04C)) //  Index compare register 1
#define QEI_INXCMP2   (*((volatile dword *) 0x400BC050)) //  Index compare register 2
#define QEI_LOAD      (*((volatile dword *) 0x400BC028)) //  Velocity timer reload register
#define QEI_TIME      (*((volatile dword *) 0x400BC02C)) //  Velocity timer register RO
#define QEI_VEL       (*((volatile dword *) 0x400BC030)) //  Velocity counter register RO
#define QEI_CAP       (*((volatile dword *) 0x400BC034)) //  Velocity capture register RO
#define QEI_VELCOMP   (*((volatile dword *) 0x400BC038)) //  Velocity compare register 0 616
#define QEI_FILTERPHA (*((volatile dword *) 0x400BC03C)) //  Digital filter register on PHA 0 617
#define QEI_FILTERPHB (*((volatile dword *) 0x400BC040)) //  Digital filter register on PHB 0 618
#define QEI_FILTERINX (*((volatile dword *) 0x400BC044)) //  Digital filter register on IDX 0 619
#define QEI_WINDOW    (*((volatile dword *) 0x400BC048)) //  Index acceptance window register 0xF 620

/* Interrupt registers */
#define QEI_INTSTAT   (*((volatile dword *) 0x400BCFE0)) //  Interrupt status register RO
#define QEI_SET       (*((volatile dword *) 0x400BCFEC)) //  Interrupt status set register WO
#define QEI_CLR       (*((volatile dword *) 0x400BCFE8)) //  Interrupt status clear register WO
#define QEI_IE        (*((volatile dword *) 0x400BCFE4)) //  Interrupt enable register RO
#define QEI_IES       (*((volatile dword *) 0x400BCFDC)) //  Interrupt enable set register WO
#define QEI_IEC       (*((volatile dword *) 0x400BCFD8)) //  Interrupt enable clear register WO


/************ LPC4088  Real Time Clock (RTC) *********************************/
/* base address 0x40024000 */
/* Miscellaneous registers */
#define RTC_ILR  (*((volatile dword *) 0x40024000)) //  Interrupt Location Register
#define RTC_CCR  (*((volatile dword *) 0x40024008)) //  Clock Control Register NC 
#define RTC_CIIR (*((volatile dword *) 0x4002400C)) //  Counter Increment Interrupt Register
#define RTC_AMR  (*((volatile dword *) 0x40024010)) //  Alarm Mask Register

/* Consolidated time registers */
#define RTC_CTIME0 (*((volatile dword *) 0x40024014)) //  Consolidated Time Register 0 NC RO
#define RTC_CTIME1 (*((volatile dword *) 0x40024018)) //  Consolidated Time Register 1 NC RO
#define RTC_CTIME2 (*((volatile dword *) 0x4002401C)) //  Consolidated Time Register 2 NC RO

/* Time counter registers */
#define RTC_SEC         (*((volatile dword *) 0x40024020)) //  Seconds Counter NC
#define RTC_MIN         (*((volatile dword *) 0x40024024)) //  Minutes Register NC 
#define RTC_HRS         (*((volatile dword *) 0x40024028)) //  Hours Register NC 
#define RTC_DOM         (*((volatile dword *) 0x4002402C)) //  Day of Month Register NC 
#define RTC_DOW         (*((volatile dword *) 0x40024030)) //  Day of Week Register NC 
#define RTC_DOY         (*((volatile dword *) 0x40024034)) //  Day of Year Register NC 
#define RTC_MONTH       (*((volatile dword *) 0x40024038)) //  Months Register NC 
#define RTC_YEAR        (*((volatile dword *) 0x4002403C)) //  Years Register NC 
#define RTC_CALIBRATION (*((volatile dword *) 0x40024040)) //  Calibration Value Register NC 

/* General purpose registers */
#define RTC_GPREG0    (*((volatile dword *) 0x40024044)) //  General Purpose Register 0 NC
#define RTC_GPREG1    (*((volatile dword *) 0x40024048)) //  General Purpose Register 1 NC
#define RTC_GPREG2    (*((volatile dword *) 0x4002404C)) //  General Purpose Register 2 NC
#define RTC_GPREG3    (*((volatile dword *) 0x40024050)) //  General Purpose Register 3 NC
#define RTC_GPREG4    (*((volatile dword *) 0x40024054)) //  General Purpose Register 4 NC
#define RTC_RTC_AUX   (*((volatile dword *) 0x4002405C)) //  RTC Auxiliary control register
#define RTC_RTC_AUXEN (*((volatile dword *) 0x40024058)) //  RTC Auxiliary Enable register 

/* Alarm register group */
#define RTC_ASEC  (*((volatile dword *) 0x40024060)) //  Alarm value for Seconds NC
#define RTC_AMIN  (*((volatile dword *) 0x40024064)) //  Alarm value for Minutes NC
#define RTC_AHRS  (*((volatile dword *) 0x40024068)) //  Alarm value for Hours NC 
#define RTC_ADOM  (*((volatile dword *) 0x4002406C)) //  Alarm value for Day of Month NC
#define RTC_ADOW  (*((volatile dword *) 0x40024070)) //  Alarm value for Day of Week NC 
#define RTC_ADOY  (*((volatile dword *) 0x40024074)) //  Alarm value for Day of Year NC 
#define RTC_AMON  (*((volatile dword *) 0x40024078)) //  Alarm value for Months NC 
#define RTC_AYRS  (*((volatile dword *) 0x4002407C)) //  Alarm value for Year NC


/************ LPC4088  Event Monitor/Recorder ********************************/
/* base address 0x40024000 */
#define ERCONTROL     (*((volatile dword *) 0x40024084)) //  Control register
#define ERSTATUS      (*((volatile dword *) 0x40024080)) //  Status register
#define ERCOUNTERS    (*((volatile dword *) 0x40024088)) //  Counters register RO
#define ERFIRSTSTAMP0 (*((volatile dword *) 0x40024090)) //  First Stamp register for channel 0. RO
#define ERFIRSTSTAMP1 (*((volatile dword *) 0x40024094)) //  First Stamp register for channel 1  RO
#define ERFIRSTSTAMP2 (*((volatile dword *) 0x40024098)) //  First Stamp register for channel 2  RO
#define ERLASTSTAMP0  (*((volatile dword *) 0x400240A0)) //  Last Stamp register for channel 0.  RO
#define ERLASTSTAMP1  (*((volatile dword *) 0x400240A4)) //  Last Stamp register for channel 1   RO
#define ERLASTSTAMP2  (*((volatile dword *) 0x400240A8)) //  Last Stamp register for channel 2   RO


/************ LPC4088  Windowed Watchdog Timer (WWDT) ************************/
/* base address 0x40000000 */
#define WWDT_MOD     (*((volatile dword *) 0x40000000)) //  Watchdog mode register
#define WWDT_TC      (*((volatile dword *) 0x40000004)) //  Watchdog timer constant register
#define WWDT_FEED    (*((volatile dword *) 0x40000008)) //  Watchdog feed sequence register. WO Write 0xAA followed by 0x55
#define WWDT_TV      (*((volatile dword *) 0x4000000C)) //  Watchdog timer value register. RO
#define WWDT_WARNINT (*((volatile dword *) 0x40000014)) //  Watchdog Warning Interrupt compare value
#define WWDT_WINDOW  (*((volatile dword *) 0x40000018)) //  Watchdog Window compare value


/************ LPC4088  Analog-to-Digital Converter (ADC) *********************/
/* base address 0x40034000 */
#define ADC_CR    (*((volatile dword *) 0x40034000)) //  Control Register
#define ADC_GDR   (*((volatile dword *) 0x40034004)) //  Global Data Register
#define ADC_INTEN (*((volatile dword *) 0x4003400C)) //  Interrupt Enable Register
#define ADC_DR0   (*((volatile dword *) 0x40034010)) //  Channel 0 Data Register RO
#define ADC_DR1   (*((volatile dword *) 0x40034014)) //  Channel 1 Data Register RO
#define ADC_DR2   (*((volatile dword *) 0x40034018)) //  Channel 2 Data Register RO
#define ADC_DR3   (*((volatile dword *) 0x4003401C)) //  Channel 3 Data Register RO
#define ADC_DR4   (*((volatile dword *) 0x40034020)) //  Channel 4 Data Register RO
#define ADC_DR5   (*((volatile dword *) 0x40034024)) //  Channel 5 Data Register RO
#define ADC_DR6   (*((volatile dword *) 0x40034028)) //  Channel 6 Data Register RO
#define ADC_DR7   (*((volatile dword *) 0x4003402C)) //  Channel 7 Data Register RO
#define ADC_STAT  (*((volatile dword *) 0x40034030)) //  Status Register RO
#define ADC_TRM   (*((volatile dword *) 0x40034034)) //  ADC trim register


/************ LPC4088  Digital-to-Analog Converter (DAC) *********************/
/* base address 0x4008C000 */
#define DAC_CR     (*((volatile dword *) 0x4008C000)) //  D/A Converter Register
#define DAC_CTRL   (*((volatile dword *) 0x4008C004)) //  DAC Control register
#define DAC_CNTVAL (*((volatile dword *) 0x4008C008)) //  DAC Counter Value register


/************ LPC4088  Comparators ******************************************/
/* base address 0x40020000 */
#define CMP_CTRL   (*((volatile dword *) 0x40020000)) //  Comparator block control register
#define CMP_CTRL0  (*((volatile dword *) 0x40020004)) //  Comparator 0 control register
#define CMP_CTRL1  (*((volatile dword *) 0x40020008)) //  Comparator 1 control register


/************ LPC4088  General Purpose DMA controller ************************/
/* base address 0x20080000 */
/* General registers */
#define DMA_INTSTAT       (*((volatile dword *) 0x20080000)) //  DMA Interrupt Status Register RO
#define DMA_INTTCSTAT     (*((volatile dword *) 0x20080004)) //  DMA Interrupt Terminal Count Request Status Register RO
#define DMA_INTTCCLEAR    (*((volatile dword *) 0x20080008)) //  DMA Interrupt Terminal Count Request Clear Register WO
#define DMA_INTERRSTAT    (*((volatile dword *) 0x2008000C)) //  DMA Interrupt Error Status Register  RO
#define DMA_INTERRCLR     (*((volatile dword *) 0x20080010)) //  DMA Interrupt Error Clear Register WO
#define DMA_RAWINTTCSTAT  (*((volatile dword *) 0x20080014)) //  DMA Raw Interrupt Terminal Count Status Register RO
#define DMA_RAWINTERRSTAT (*((volatile dword *) 0x20080018)) //  DMA Raw Error Interrupt Status Register  RO
#define DMA_ENBLDCHNS     (*((volatile dword *) 0x2008001C)) //  DMA Enabled Channel Register  RO
#define DMA_SOFTBREQ      (*((volatile dword *) 0x20080020)) //  DMA Software Burst Request Register 
#define DMA_SOFTSREQ      (*((volatile dword *) 0x20080024)) //  DMA Software Single Request Register
#define DMA_SOFTLBREQ     (*((volatile dword *) 0x20080028)) //  DMA Software Last Burst Request Register
#define DMA_SOFTLSREQ     (*((volatile dword *) 0x2008002C)) //  DMA Software Last Single Request Register
#define DMA_CONFIG        (*((volatile dword *) 0x20080030)) //  DMA Configuration Register
#define DMA_SYNC          (*((volatile dword *) 0x20080034)) //  DMA Synchronization Register

/* Channel 0 registers */
#define DMA_SRCADDR0  (*((volatile dword *) 0x20080100)) //  DMA Channel 0 Source Address Register
#define DMA_DESTADDR0 (*((volatile dword *) 0x20080104)) //  DMA Channel 0 Destination Address Register
#define DMA_LLI0      (*((volatile dword *) 0x20080108)) //  DMA Channel 0 Linked List Item Register
#define DMA_CONTROL0  (*((volatile dword *) 0x2008010C)) //  DMA Channel 0 Control Register
#define DMA_CONFIG0   (*((volatile dword *) 0x20080110)) //  DMA Channel 0 Configuration Register

/* Channel 1 registers */
#define DMA_SRCADDR1  (*((volatile dword *) 0x20080120)) //  DMA Channel 1 Source Address Register
#define DMA_DESTADDR1 (*((volatile dword *) 0x20080124)) //  DMA Channel 1 Destination Address Register
#define DMA_LLI1      (*((volatile dword *) 0x20080128)) //  DMA Channel 1 Linked List Item Register
#define DMA_CONTROL1  (*((volatile dword *) 0x2008012C)) //  DMA Channel 1 Control Register
#define DMA_CONFIG1   (*((volatile dword *) 0x20080130)) //  DMA Channel 1 Configuration Register

/* Channel 2 registers */
#define DMA_SRCADDR2  (*((volatile dword *) 0x20080140)) //  DMA Channel 2 Source Address Register 
#define DMA_DESTADDR2 (*((volatile dword *) 0x20080144)) //  DMA Channel 2 Destination Address Register
#define DMA_LLI2      (*((volatile dword *) 0x20080148)) //  DMA Channel 2 Linked List Item Register
#define DMA_CONTROL2  (*((volatile dword *) 0x2008014C)) //  DMA Channel 2 Control Register
#define DMA_CONFIG2   (*((volatile dword *) 0x20080150)) //  DMA Channel 2 Configuration Register

/* Channel 3 registers */
#define DMA_SRCADDR3  (*((volatile dword *) 0x20080160)) //  DMA Channel 3 Source Address Register
#define DMA_DESTADDR3 (*((volatile dword *) 0x20080164)) //  DMA Channel 3 Destination Address Register
#define DMA_LLI3      (*((volatile dword *) 0x20080168)) //  DMA Channel 3 Linked List Item Register
#define DMA_CONTROL3  (*((volatile dword *) 0x2008016C)) //  DMA Channel 3 Control Register
#define DMA_CONFIG3   (*((volatile dword *) 0x20080170)) //  DMA Channel 3 Configuration Register

/* Channel 4 registers */
#define DMA_SRCADDR4  (*((volatile dword *) 0x20080180)) //  DMA Channel 4 Source Address Register 
#define DMA_DESTADDR4 (*((volatile dword *) 0x20080184)) //  DMA Channel 4 Destination Address Register
#define DMA_LLI4      (*((volatile dword *) 0x20080188)) //  DMA Channel 4 Linked List Item Register 
#define DMA_CONTROL4  (*((volatile dword *) 0x2008018C)) //  DMA Channel 4 Control Register
#define DMA_CONFIG4   (*((volatile dword *) 0x20080190)) //  DMA Channel 4 Configuration Register

/* Channel 5 registers */
#define DMA_SRCADDR5  (*((volatile dword *) 0x200801A0)) //  DMA Channel 5 Source Address Register
#define DMA_DESTADDR5 (*((volatile dword *) 0x200801A4)) //  DMA Channel 5 Destination Address Register
#define DMA_LLI5      (*((volatile dword *) 0x200801A8)) //  DMA Channel 5 Linked List Item Register
#define DMA_CONTROL5  (*((volatile dword *) 0x200801AC)) //  DMA Channel 5 Control Register
#define DMA_CONFIG5   (*((volatile dword *) 0x200801B0)) //  DMA Channel 5 Configuration Register

/* Channel 6 registers */
#define DMA_SRCADDR6  (*((volatile dword *) 0x200801C0)) //  DMA Channel 6 Source Address Register
#define DMA_DESTADDR6 (*((volatile dword *) 0x200801C4)) //  DMA Channel 6 Destination Address Register
#define DMA_LLI6      (*((volatile dword *) 0x200801C8)) //  DMA Channel 6 Linked List Item Register
#define DMA_CONTROL6  (*((volatile dword *) 0x200801CC)) //  DMA Channel 6 Control Register
#define DMA_CONFIG6   (*((volatile dword *) 0x200801D0)) //  DMA Channel 6 Configuration Register

/* Channel 7 registers */
#define DMA_SRCADDR7  (*((volatile dword *) 0x200801E0)) //  DMA Channel 7 Source Address Register 
#define DMA_DESTADDR7 (*((volatile dword *) 0x200801E4)) //  DMA Channel 7 Destination Address Register
#define DMA_LLI7      (*((volatile dword *) 0x200801E8)) //  DMA Channel 7 Linked List Item Register
#define DMA_CONTROL7  (*((volatile dword *) 0x200801EC)) //  DMA Channel 7 Control Register
#define DMA_CONFIG7   (*((volatile dword *) 0x200801F0)) //  DMA Channel 7 Configuration Register


/************ LPC4088  CRC engine ********************************************/
/* base address 0x20090000 */
#define CRC_MODE      (*((volatile dword *) 0x20090000)) //  CRC mode register 
#define CRC_SEED      (*((volatile dword *) 0x20090004)) //  CRC seed register 
#define CRC_SUM       (*((volatile dword *) 0x20090008)) //  CRC checksum register RO
#define CRC_DATA      (*((volatile dword *) 0x20090008)) //  CRC data register WO


/************ LPC4088  EEPROM memory *****************************************/
/* base address 0x00200000 */
/* EEPROM registers */
#define EE_CMD        (*((volatile dword *) 0x00200080)) //  EEPROM command register 
#define EE_ADDR       (*((volatile dword *) 0x00200084)) //  EEPROM address register 
#define EE_WDATA      (*((volatile dword *) 0x00200088)) //  EEPROM write data register WO
#define EE_RDATA      (*((volatile dword *) 0x0020008C)) //  EEPROM read data register RO
#define EE_WSTATE     (*((volatile dword *) 0x00200090)) //  EEPROM wait state register 
#define EE_CLKDIV     (*((volatile dword *) 0x00200094)) //  EEPROM clock divider register
#define EE_PWRDWN     (*((volatile dword *) 0x00200098)) //  EEPROM power-down register 

/* EEPROM interrupt registers */
#define EE_INTSTAT    (*((volatile dword *) 0x00200FE0)) //  EEPROM interrupt status RO
#define EE_INTSTATCLR (*((volatile dword *) 0x00200FE8)) //  EEPROM interrupt status clear WO
#define EE_INTSTATSET (*((volatile dword *) 0x00200FEC)) //  EEPROM interrupt status set WO
#define EE_INTEN      (*((volatile dword *) 0x00200FE4)) //  EEPROM interrupt enable RO
#define EE_INTENCLR   (*((volatile dword *) 0x00200FD8)) //  EEPROM interrupt enable clear WO
#define EE_INTENSET   (*((volatile dword *) 0x00200FDC)) //  EEPROM interrupt enable set WO


/************ LPC4088  Flash memory ******************************************/
/* base address 0x00200000 */
#define FMSSTART   (*((volatile dword *) 0x00200020)) //  Signature start address register
#define FMSSTOP    (*((volatile dword *) 0x00200024)) //  Signature stop-address register
#define FMSW0      (*((volatile dword *) 0x0020002C)) //  128-bit signature Word 0  RO
#define FMSW1      (*((volatile dword *) 0x00200030)) //  128-bit signature Word 1  RO 
#define FMSW2      (*((volatile dword *) 0x00200034)) //  128-bit signature Word 2  RO
#define FMSW3      (*((volatile dword *) 0x00200038)) //  128-bit signature Word 3  RO
/*  Status, StatusClear in EE_INTSTAT, EE_INTSTATCLR */


/************ LPC4088  JTAG, Debug, and Trace ********************************/
#define MEMMAP     (*((volatile dword *) 0x400FC040)) // Memory Mapping Control register




#endif  
